@@ -222,10 +222,6 @@ void gattrib_main(void *closure, int argc, char *argv[])
   /* Initialize gEDA stuff */
   libgeda_init();
 
-  /* Ensure object->sel_func can be used to correctly determine object
-   * locking when the project is saved out */
-  select_func = s_toplevel_select_object;
-
   /* Note that argv_index holds index to first non-flag command line option 
    * (that is, to the first file name) */
   argv_index = parse_commandline(argc, argv);
@@ -251,7 +247,7 @@ void gattrib_main(void *closure, int argc, char *argv[])
   pr_current = s_toplevel_new();
 
   /* ----- Read in RC files.   ----- */
-  g_rc_parse(pr_current, "gattribrc", NULL);
+  g_rc_parse (pr_current, argv[0], "gattribrc", NULL);
 
   i_vars_set(pr_current);
 
