@@ -279,7 +279,7 @@ void s_rename_all_lowlevel(NETLIST * netlist_head, char *src, char *dest)
 
 }
 
-void s_rename_all(TOPLEVEL * pr_current, NETLIST * netlist_head)
+void s_rename_all (TOPLEVEL *toplevel, NETLIST * netlist_head)
 {
     int i;
 
