@@ -254,7 +254,7 @@ void x_dialog_unsaved_data()
         }
       case GTK_RESPONSE_YES:
         {
-          s_toplevel_gtksheet_to_toplevel();  /* Dumps sheet data into TOPLEVEL */
+          s_toplevel_gtksheet_to_toplevel(pr_current);  /* Dumps sheet data into TOPLEVEL */
           s_page_save_all(pr_current);  /* saves all pages in design */
           sheet_head->CHANGED = FALSE;
           gattrib_quit(0);
