@@ -36,8 +36,6 @@
 #endif
 
 #define DEFAULT_UNTITLED_NAME    "untitled"
-#define DEFAULT_SCHEME_DIRECTORY "./"
-#define DEFAULT_FONT_DIRECTORY   "../lib/sym/font"
 #define DEFAULT_BITMAP_DIRECTORY   "non-existant"
 #define DEFAULT_HIERARCHY_NETNAME_SEPARATOR "/"
 #define DEFAULT_HIERARCHY_NETATTRIB_SEPARATOR "/"
