@@ -14,7 +14,9 @@ schbasename=`basename $INPUT .sch`
 SCMDIR=$SRCDIR/../scheme \
 SYMDIR=$SRCDIR/../../symbols \
 GEDADATARC=$BUILDDIR/../lib \
-../src/gnetlist -o ${BUILDDIR}/new_${schbasename}.$BACKEND -g $BACKEND $INPUT 
+../src/gnetlist -L ${SRCDIR}/../../libgeda/scheme \
+  -L ${BUILDDIR}/../../libgeda/scheme \
+  -o ${BUILDDIR}/new_${schbasename}.$BACKEND -g $BACKEND $INPUT
 status=$?
 
 if [ "$status" != 0 ] 
