@@ -2,6 +2,7 @@
  * gEDA - GPL Electronic Design Automation
  * gschem - gEDA Schematic Capture
  * Copyright (C) 1998-2010 Ales Hvezda
+ * Copyright (C) 1998-2011 gEDA Contributors (see ChangeLog for details)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,18 +22,18 @@
 #ifndef H_GSCHEM_GLOBALS_H
 #define H_GSCHEM_GLOBALS_H
 
-/* used by various guile functions, set in x_event* functions */
-extern GSCHEM_TOPLEVEL *global_window_current;
 
 /* window list */
 extern GList *global_window_list;
 
+/* Manager for recently used files */
+GtkRecentManager *recent_manager;
+
 /* colors */
 extern GdkColor white;
 extern GdkColor black;
 
 extern char *rc_filename; 
-extern char *script_filename;
 extern char *output_filename;
 
 
@@ -43,7 +44,6 @@ extern int logging_dest;
 /* command line options */
 extern int quiet_mode;
 extern int verbose_mode;
-extern int stroke_info_mode;
 extern int auto_place_mode;
 
 #define MAX_BUFFERS 	5
@@ -51,23 +51,7 @@ extern int auto_place_mode;
 extern GList *object_buffer[MAX_BUFFERS];
 
 /* Hooks */
-extern SCM add_component_hook;
-extern SCM add_component_object_hook;
-extern SCM mirror_component_object_hook;
-extern SCM rotate_component_object_hook;
 extern SCM complex_place_list_changed_hook;
-extern SCM copy_component_hook;
-extern SCM move_component_hook;
-extern SCM add_pin_hook;
-extern SCM mirror_pin_hook;
-extern SCM rotate_pin_hook;
-extern SCM add_attribute_hook;
-extern SCM deselect_component_hook;
-extern SCM deselect_net_hook;
-extern SCM deselect_all_hook;
-extern SCM select_component_hook;
-extern SCM select_net_hook;
-extern SCM new_page_hook;
 
 #include "gettext.h"
 #ifdef ENABLE_NLS
