@@ -1,7 +1,7 @@
 /* gEDA - GPL Electronic Design Automation
  * gschem - gEDA Schematic Capture
  * Copyright (C) 1998-2010 Ales Hvezda
- * Copyright (C) 1998-2010 gEDA Contributors (see ChangeLog for details)
+ * Copyright (C) 1998-2011 gEDA Contributors (see ChangeLog for details)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -57,45 +57,13 @@ void o_copy_start(GSCHEM_TOPLEVEL *w_current, int w_x, int w_y)
 
   toplevel->page_current->place_list =
     o_glist_copy_all (toplevel, s_current,
-                      toplevel->page_current->place_list,
-                      SELECTION_FLAG);
+                      toplevel->page_current->place_list);
 
   w_current->inside_action = 1;
   i_set_state(w_current, COPY);
   o_place_start (w_current, w_x, w_y);
 }
 
-
-/*! \todo Finish function documentation!!!
- *  \brief
- *  \par Function Description
- *
- */
-static void o_copy_end_generic(GSCHEM_TOPLEVEL *w_current, int multiple)
-{
-  GList *new_objects;
-  GList *iter;
-  OBJECT *object;
-
-  o_place_end (w_current, w_current->second_wx, w_current->second_wy, multiple, &new_objects);
-
-  /* Run the copy component hook for all new components */
-  for (iter = new_objects;
-       iter != NULL;
-       iter = g_list_next (iter)) {
-    object = iter->data;
-    if ( (object->type == OBJ_COMPLEX) &&
-         (scm_hook_empty_p(copy_component_hook) == SCM_BOOL_F)) {
-      scm_run_hook(copy_component_hook,
-                   scm_cons (g_make_attrib_smob_list(w_current, object),
-                   SCM_EOL));
-    }
-  }
-
-  g_list_free (new_objects);
-}
-
-
 /*! \todo Finish function documentation!!!
  *  \brief
  *  \par Function Description
@@ -103,7 +71,8 @@ static void o_copy_end_generic(GSCHEM_TOPLEVEL *w_current, int multiple)
  */
 void o_copy_end(GSCHEM_TOPLEVEL *w_current)
 {
-  o_copy_end_generic (w_current, FALSE);
+  o_place_end (w_current, w_current->second_wx, w_current->second_wy, FALSE,
+               NULL, "%paste-objects-hook");
 }
 
 
@@ -114,5 +83,6 @@ void o_copy_end(GSCHEM_TOPLEVEL *w_current)
  */
 void o_copy_multiple_end(GSCHEM_TOPLEVEL *w_current)
 {
-  o_copy_end_generic (w_current, TRUE);
+  o_place_end (w_current, w_current->second_wx, w_current->second_wy, TRUE,
+               NULL, "%paste-objects-hook");
 }
