@@ -193,8 +193,7 @@ x_clipboard_set (GSCHEM_TOPLEVEL *w_current, const GList *object_list)
 
   /* Copy the objects to the clipboard buffer */
   w_current->clipboard_buffer =
-    o_glist_copy_all (toplevel, object_list, w_current->clipboard_buffer,
-                      SELECTION_FLAG);
+    o_glist_copy_all (toplevel, object_list, w_current->clipboard_buffer);
 
   /* Advertise that the data is available */
   result = gtk_clipboard_set_with_data (cb, &target, 1,
@@ -225,6 +224,7 @@ x_clipboard_get (GSCHEM_TOPLEVEL *w_current)
   GtkSelectionData *selection_data;
   GList *object_list = NULL;
   const guchar *buf;
+  GError * err = NULL;
 
   /* Try to get the contents of the clipboard */
   selection_data = gtk_clipboard_wait_for_contents (cb, type);
@@ -237,11 +237,23 @@ x_clipboard_get (GSCHEM_TOPLEVEL *w_current)
   buf = selection_data->data;
 #endif
 
-  toplevel->ADDING_SEL = 1; /* HACK: Avoid adding objects to the tile system */
   object_list = o_read_buffer (toplevel, object_list,
-                               (gchar *) buf, -1, "Clipboard");
-  toplevel->ADDING_SEL = 0;
+                               (gchar *) buf, -1, "Clipboard", &err);
 
+  if (err) {
+    GtkWidget * dialog = gtk_message_dialog_new_with_markup
+      (GTK_WINDOW (w_current->main_window),
+       GTK_DIALOG_DESTROY_WITH_PARENT,
+       GTK_MESSAGE_ERROR,
+       GTK_BUTTONS_OK,
+       _("<b>Invalid schematic on clipboard.</b>\n\nAn error occurred while inserting clipboard data: %s."),
+       err->message);
+    gtk_window_set_title (GTK_WINDOW (dialog), _("Clipboard insertion failed"));
+
+     gtk_dialog_run (GTK_DIALOG (dialog));
+     gtk_widget_destroy (dialog);
+     g_error_free(err);
+  }
   gtk_selection_data_free (selection_data);
   return object_list;
 }
