@@ -1,7 +1,7 @@
 /* gEDA - GPL Electronic Design Automation
  * gschem - gEDA Schematic Capture
  * Copyright (C) 1998-2010 Ales Hvezda
- * Copyright (C) 1998-2010 gEDA Contributors (see ChangeLog for details)
+ * Copyright (C) 1998-2011 gEDA Contributors (see ChangeLog for details)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -59,9 +59,7 @@ gint x_event_expose(GtkWidget *widget, GdkEventExpose *event,
   printf("EXPOSE\n");
 #endif
 
-  exit_if_null(w_current);
-  /* nasty global variable */
-  global_window_current = w_current;
+  g_return_val_if_fail ((w_current != NULL), 0);
 
   save_cr = w_current->cr;
   save_pl = w_current->pl;
@@ -96,12 +94,13 @@ gint x_event_button_pressed(GtkWidget *widget, GdkEventButton *event,
                             GSCHEM_TOPLEVEL *w_current)
 {
   TOPLEVEL *toplevel = w_current->toplevel;
-  int prev_state;
   int w_x, w_y;
   int unsnapped_wx, unsnapped_wy;
 
-  exit_if_null(w_current);
-  global_window_current = w_current;
+  g_return_val_if_fail ((w_current != NULL), 0);
+
+  scm_dynwind_begin (0);
+  g_dynwind_window (w_current);
 
 #if DEBUG
   printf("pressed button %d! \n", event->button);
@@ -124,6 +123,7 @@ gint x_event_button_pressed(GtkWidget *widget, GdkEventButton *event,
     if (o_select_selected (w_current)) {
        o_edit(w_current, geda_list_get_glist( toplevel->page_current->selection_list ));
        i_set_state(w_current, SELECT);
+       scm_dynwind_end ();
        return(0);
     }
   }
@@ -132,6 +132,10 @@ gint x_event_button_pressed(GtkWidget *widget, GdkEventButton *event,
   w_current->CONTROLKEY = (event->state & GDK_CONTROL_MASK) ? 1 : 0;
   w_current->ALTKEY     = (event->state & GDK_MOD1_MASK) ? 1 : 0;
 
+  /* Huge switch statement to evaluate state transitions. Jump to
+   * end_button_pressed label to escape the state evaluation rather than
+   * returning from the function directly. */
+
   if (event->button == 1) {
     switch(w_current->event_state) {
 
@@ -291,7 +295,8 @@ gint x_event_button_pressed(GtkWidget *widget, GdkEventButton *event,
         }
         break;
       case(ENDCOMP):
-        o_complex_end(w_current, w_x, w_y, w_current->continue_component_place);
+        o_place_end(w_current, w_x, w_y, w_current->continue_component_place,
+                    NULL, "%add-objects-hook");
         if (!w_current->continue_component_place) {
           w_current->inside_action = 0;
           i_set_state(w_current, SELECT);
@@ -300,18 +305,15 @@ gint x_event_button_pressed(GtkWidget *widget, GdkEventButton *event,
         break;
 
       case(ENDPASTE):
-        o_place_end(w_current, w_x, w_y, FALSE, NULL);
+        o_place_end(w_current, w_x, w_y, FALSE, NULL, "%paste-objects-hook");
         w_current->inside_action = 0;
         i_set_state(w_current, SELECT);
         i_update_toolbar(w_current);
         break;
 
       case(ENDROTATEP):
-        prev_state = toplevel->DONT_REDRAW;
-        toplevel->DONT_REDRAW = 0;
         o_rotate_world_update(w_current, w_x, w_y, 90,
           geda_list_get_glist(toplevel->page_current->selection_list ));
-        toplevel->DONT_REDRAW = prev_state;
 
         w_current->inside_action = 0;
         i_set_state(w_current, SELECT);
@@ -329,7 +331,7 @@ gint x_event_button_pressed(GtkWidget *widget, GdkEventButton *event,
         break;
 
       case(ENDTEXT):
-        o_place_end(w_current, w_x, w_y, FALSE, NULL);
+        o_place_end(w_current, w_x, w_y, FALSE, NULL, "%add-objects-hook");
         w_current->inside_action = 0;
         i_set_state(w_current, SELECT);
         i_update_toolbar(w_current);
@@ -353,17 +355,15 @@ gint x_event_button_pressed(GtkWidget *widget, GdkEventButton *event,
 
     /* try this out and see how it behaves */
     if (w_current->inside_action) {
-      if (w_current->event_state == ENDCOMP ||
-          w_current->event_state == ENDTEXT ||
-          w_current->event_state == ENDMOVE ||
-          w_current->event_state == ENDCOPY ||
-          w_current->event_state == ENDMCOPY ||
-          w_current->event_state == ENDPASTE ) {
-            return(0);
-          } else {
-            i_callback_cancel(w_current, 0, NULL);
-            return(0);
-          }
+      if (!(w_current->event_state == ENDCOMP ||
+            w_current->event_state == ENDTEXT ||
+            w_current->event_state == ENDMOVE ||
+            w_current->event_state == ENDCOPY ||
+            w_current->event_state == ENDMCOPY ||
+            w_current->event_state == ENDPASTE )) {
+        i_callback_cancel(w_current, 0, NULL);
+      }
+      goto end_button_pressed;
     }
 
     switch(w_current->middle_button) {
@@ -394,7 +394,7 @@ gint x_event_button_pressed(GtkWidget *widget, GdkEventButton *event,
         w_current->inside_action = 0;
         i_set_state(w_current, SELECT);
         i_update_toolbar(w_current);
-        return(0);
+        goto end_button_pressed;
       }
 
       if (w_current->ALTKEY) {
@@ -510,6 +510,10 @@ gint x_event_button_pressed(GtkWidget *widget, GdkEventButton *event,
       i_update_toolbar(w_current);
     }
   }
+
+ end_button_pressed:
+  scm_dynwind_end ();
+
   return(0);
 }
 
@@ -521,13 +525,9 @@ gint x_event_button_pressed(GtkWidget *widget, GdkEventButton *event,
 gint x_event_button_released(GtkWidget *widget, GdkEventButton *event,
                              GSCHEM_TOPLEVEL *w_current)
 {
-  TOPLEVEL *toplevel = w_current->toplevel;
-  int prev_state;
-  int w_x, w_y;
   int unsnapped_wx, unsnapped_wy;
 
-  exit_if_null(w_current);
-  global_window_current = w_current;
+  g_return_val_if_fail ((w_current != NULL), 0);
 
 #if DEBUG
   printf("released! %d \n", w_current->event_state);
@@ -539,8 +539,12 @@ gint x_event_button_released(GtkWidget *widget, GdkEventButton *event,
 
   SCREENtoWORLD (w_current, (int) event->x, (int) event->y,
                  &unsnapped_wx, &unsnapped_wy);
-  w_x = snap_grid (w_current, unsnapped_wx);
-  w_y = snap_grid (w_current, unsnapped_wy);
+
+  /* Huge switch statement to evaluate state transitions. Jump to
+   * end_button_released label to escape the state evaluation rather
+   * than returning from the function directly. */
+  scm_dynwind_begin (0);
+  g_dynwind_window (w_current);
 
   if (event->button == 1) {
     switch(w_current->event_state) {
@@ -639,12 +643,7 @@ gint x_event_button_released(GtkWidget *widget, GdkEventButton *event,
         o_place_rotate(w_current);
 
         if (w_current->event_state == ENDCOMP) {
-          /* Run the complex place list changed hook without redrawing */
-          /* since all objects are being redrawn afterwards */
-          prev_state = toplevel->DONT_REDRAW;
-          toplevel->DONT_REDRAW = 1;
           o_complex_place_changed_run_hook (w_current);
-          toplevel->DONT_REDRAW = prev_state;
         }
 
         if (w_current->event_state == ENDMOVE) {
@@ -653,7 +652,7 @@ gint x_event_button_released(GtkWidget *widget, GdkEventButton *event,
           o_place_invalidate_rubber (w_current, TRUE);
         }
         w_current->rubber_visible = 1;
-        return(0);
+        goto end_button_released;
       }
     }
 
@@ -714,6 +713,9 @@ gint x_event_button_released(GtkWidget *widget, GdkEventButton *event,
       i_update_toolbar(w_current);
     }
   }
+ end_button_released:
+  scm_dynwind_end ();
+
   return(0);
 }
 
@@ -731,8 +733,7 @@ gint x_event_motion(GtkWidget *widget, GdkEventMotion *event,
   int skip_event=0;
   GdkEvent *test_event;
 
-  exit_if_null(w_current);
-  global_window_current = w_current;
+  g_return_val_if_fail ((w_current != NULL), 0);
 
   w_current->SHIFTKEY   = (event->state & GDK_SHIFT_MASK  ) ? 1 : 0;
   w_current->CONTROLKEY = (event->state & GDK_CONTROL_MASK) ? 1 : 0;
@@ -772,7 +773,6 @@ gint x_event_motion(GtkWidget *widget, GdkEventMotion *event,
   if (w_current->cowindow) {
     coord_display_update(w_current, (int) event->x, (int) event->y);
   }
-
   if (w_current->third_button == MOUSEPAN_ENABLED || w_current->middle_button == MID_MOUSEPAN_ENABLED) {
     if((w_current->event_state == MOUSEPAN) &&
        w_current->inside_action) {
@@ -791,6 +791,12 @@ gint x_event_motion(GtkWidget *widget, GdkEventMotion *event,
        }
   }
 
+  /* Huge switch statement to evaluate state transitions. Jump to
+   * end_motion label to escape the state evaluation rather
+   * than returning from the function directly. */
+  scm_dynwind_begin (0);
+  g_dynwind_window (w_current);
+
   switch(w_current->event_state) {
 
     case(SELECT):
@@ -802,22 +808,27 @@ gint x_event_motion(GtkWidget *widget, GdkEventMotion *event,
     break;
 
     case(STARTSELECT):
-    if ( (!w_current->drag_can_move) ||
-         (w_current->drag_can_move &&
-          (! o_find_selected_object(w_current,
-                                    w_current->first_wx, w_current->first_wy)))) {
+    /* If the shift or control keys are pressed, that means the user definately wants to drag out a
+     * selection box.  Otherwise, if there is not a selected object under the cursor, look for one
+     * that could be selected and start moving it.
+     */
+    if (w_current->SHIFTKEY || w_current->CONTROLKEY
+            || (!o_find_selected_object(w_current, w_current->first_wx, w_current->first_wy)
+                && (!o_find_object(w_current, w_current->first_wx, w_current->first_wy, TRUE)
+                    || !o_select_selected(w_current)))) {
       if (o_select_box_start(w_current, unsnapped_wx, unsnapped_wy)) {
         w_current->event_state = SBOX;
         w_current->inside_action = 1;
       }
       break;
     } else {
-      /* Start the object movement */
+      /* Start moving the selected object(s) */
       o_move_start(w_current, w_x, w_y);
       w_current->event_state = ENDMOVE;
       w_current->inside_action = 1;
+      /* Fall through bottom of case to finish the move */
     }
-    /* Fall through */
+    /* Fall through to handle move */
     case(ENDMOVE):
     case(MOVE):
     if (w_current->inside_action)
@@ -894,6 +905,9 @@ gint x_event_motion(GtkWidget *widget, GdkEventMotion *event,
     break;
 
   }
+
+  scm_dynwind_end ();
+
   return(0);
 }
 
@@ -928,7 +942,6 @@ x_event_configure (GtkWidget         *widget,
   gdouble relativ_zoom_factor = 1.0;
 
   g_assert (toplevel != NULL);
-  global_window_current = w_current;
 
   if (toplevel->page_current == NULL) {
     /* don't want to call this if the current page isn't setup yet */
@@ -991,11 +1004,9 @@ x_event_configure (GtkWidget         *widget,
   /* restore current page to saved value */
   s_page_goto (toplevel, old_page_current);
 
-  if (!toplevel->DONT_REDRAW) {
-    /* redraw the current page and update UI */
-    o_invalidate_all (w_current);
-    x_scrollbars_update (w_current);
-  }
+  /* redraw the current page and update UI */
+  o_invalidate_all (w_current);
+  x_scrollbars_update (w_current);
 
   return FALSE;
 }
@@ -1051,8 +1062,7 @@ void x_event_hschanged (GtkAdjustment *adj, GSCHEM_TOPLEVEL *w_current)
   int new_left;
   GtkAdjustment        *hadjustment;
 
-  exit_if_null(w_current);
-  global_window_current = w_current;
+  g_return_if_fail (w_current != NULL);
 
   if (w_current->scrollbars_flag == FALSE) {
     return;
@@ -1069,9 +1079,7 @@ void x_event_hschanged (GtkAdjustment *adj, GSCHEM_TOPLEVEL *w_current)
     toplevel->page_current->right -
     (current_left - new_left);
 
-  if (!toplevel->DONT_REDRAW) {
-    o_invalidate_all (w_current);
-  }
+  o_invalidate_all (w_current);
 }
 
 /*! \todo Finish function documentation!!!
@@ -1086,8 +1094,7 @@ void x_event_vschanged (GtkAdjustment *adj, GSCHEM_TOPLEVEL *w_current)
   int new_bottom;
   GtkAdjustment        *vadjustment;
 
-  exit_if_null(w_current);
-  global_window_current = w_current;
+  g_return_if_fail (w_current != NULL);
 
   if (w_current->scrollbars_flag == FALSE) {
     return;
@@ -1111,9 +1118,7 @@ void x_event_vschanged (GtkAdjustment *adj, GSCHEM_TOPLEVEL *w_current)
          toplevel->page_current->bottom);
 #endif
 
-  if (!toplevel->DONT_REDRAW) {
-    o_invalidate_all (w_current);
-  }
+  o_invalidate_all (w_current);
 }
 
 /*! \todo Finish function documentation!!!
@@ -1124,8 +1129,7 @@ void x_event_vschanged (GtkAdjustment *adj, GSCHEM_TOPLEVEL *w_current)
 gint x_event_enter(GtkWidget *widget, GdkEventCrossing *event,
                    GSCHEM_TOPLEVEL *w_current)
 {
-  exit_if_null(w_current);
-  global_window_current = w_current;
+  g_return_val_if_fail ((w_current != NULL), 0);
   /* do nothing or now */
   return(0);
 }
@@ -1166,8 +1170,6 @@ gboolean x_event_key (GtkWidget *widget, GdkEventKey *event,
   int control_key = 0;
   int pressed;
 
-  global_window_current = w_current;
-
 #if DEBUG
   printf("x_event_key_pressed: Pressed key %i.\n", event->keyval);
 #endif
@@ -1199,6 +1201,13 @@ gboolean x_event_key (GtkWidget *widget, GdkEventKey *event,
       break;
   }
 
+
+  /* Huge switch statement to evaluate state transitions. Jump to
+   * end_key label to escape the state evaluation rather
+   * than returning from the function directly. */
+  scm_dynwind_begin (0);
+  g_dynwind_window (w_current);
+
   switch (w_current->event_state) {
     case ENDLINE:
       if (control_key) {
@@ -1245,8 +1254,9 @@ gboolean x_event_key (GtkWidget *widget, GdkEventKey *event,
   }
 
   if (pressed)
-    retval = g_keys_execute (w_current, event->state, event->keyval)
-               ? TRUE : FALSE;
+    retval = g_keys_execute (w_current, event) ? TRUE : FALSE;
+
+  scm_dynwind_end ();
 
   return retval;
 }
@@ -1267,8 +1277,7 @@ gint x_event_scroll (GtkWidget *widget, GdkEventScroll *event,
   int pan_direction = 1;
   int zoom_direction = ZOOM_IN;
 
-  exit_if_null(w_current);
-  global_window_current = w_current;
+  g_return_val_if_fail ((w_current != NULL), 0);
 
   /* update the state of the modifiers */
   w_current->SHIFTKEY   = (event->state & GDK_SHIFT_MASK  ) ? 1 : 0;
