@@ -71,7 +71,7 @@ gchar *o_save_buffer (TOPLEVEL *toplevel, const GList *object_list)
 
   acc = g_string_new (o_file_format_header());
 
-  buffer = o_save_objects (object_list, FALSE);
+  buffer = o_save_objects (toplevel, object_list, FALSE);
   g_string_append (acc, buffer);
   g_free (buffer);
 
@@ -89,11 +89,12 @@ gchar *o_save_buffer (TOPLEVEL *toplevel, const GList *object_list)
  *  we recurse for saving out those attributes, the function must be called
  *  with save_attribs passed as TRUE.
  *
+ *  \param [in] toplevel      A TOPLEVEL structure.
  *  \param [in] object_list   The head of a GList of objects to save.
  *  \param [in] save_attribs  Should attribute objects encounterd be saved?
  *  \returns a buffer containing schematic data or NULL on failure.
  */
-gchar *o_save_objects (const GList *object_list, gboolean save_attribs)
+gchar *o_save_objects (TOPLEVEL *toplevel, const GList *object_list, gboolean save_attribs)
 {
   OBJECT *o_current;
   const GList *iter;
@@ -113,27 +114,27 @@ gchar *o_save_objects (const GList *object_list, gboolean save_attribs)
       switch (o_current->type) {
 
         case(OBJ_LINE):
-          out = o_line_save(o_current);
+          out = o_line_save(toplevel, o_current);
           break;
 
         case(OBJ_NET):
-          out = o_net_save(o_current);
+          out = o_net_save(toplevel, o_current);
           break;
 
         case(OBJ_BUS):
-          out = o_bus_save(o_current);
+          out = o_bus_save(toplevel, o_current);
           break;
 
         case(OBJ_BOX):
-          out = o_box_save(o_current);
+          out = o_box_save(toplevel, o_current);
           break;
 
         case(OBJ_CIRCLE):
-          out = o_circle_save(o_current);
+          out = o_circle_save(toplevel, o_current);
           break;
 
         case(OBJ_COMPLEX):
-          out = o_complex_save(o_current);
+          out = o_complex_save(toplevel, o_current);
           g_string_append_printf(acc, "%s\n", out);
           already_wrote = TRUE;
           g_free(out); /* need to free here because of the above flag */
@@ -141,7 +142,7 @@ gchar *o_save_objects (const GList *object_list, gboolean save_attribs)
           if (o_complex_is_embedded(o_current)) {
             g_string_append(acc, "[\n");
 
-            out = o_save_objects(o_current->complex->prim_objs, FALSE);
+            out = o_save_objects(toplevel, o_current->complex->prim_objs, FALSE);
             g_string_append (acc, out);
             g_free(out);
 
@@ -150,27 +151,27 @@ gchar *o_save_objects (const GList *object_list, gboolean save_attribs)
           break;
 
         case(OBJ_PLACEHOLDER):  /* new type by SDB 1.20.2005 */
-          out = o_complex_save(o_current);
+          out = o_complex_save(toplevel, o_current);
           break;
 
         case(OBJ_TEXT):
-          out = o_text_save(o_current);
+          out = o_text_save(toplevel, o_current);
           break;
 
         case(OBJ_PATH):
-          out = o_path_save(o_current);
+          out = o_path_save(toplevel, o_current);
           break;
 
         case(OBJ_PIN):
-          out = o_pin_save(o_current);
+          out = o_pin_save(toplevel, o_current);
           break;
 
         case(OBJ_ARC):
-          out = o_arc_save(o_current);
+          out = o_arc_save(toplevel, o_current);
           break;
 
         case(OBJ_PICTURE):
-          out = o_picture_save(o_current);
+          out = o_picture_save(toplevel, o_current);
           break;
 
         default:
@@ -197,7 +198,7 @@ gchar *o_save_objects (const GList *object_list, gboolean save_attribs)
       if (o_current->attribs != NULL) {
         g_string_append (acc, "{\n");
 
-        out = o_save_objects (o_current->attribs, TRUE);
+        out = o_save_objects (toplevel, o_current->attribs, TRUE);
         g_string_append (acc, out);
         g_free(out);
 
@@ -214,48 +215,40 @@ gchar *o_save_objects (const GList *object_list, gboolean save_attribs)
 /*! \brief Save a file
  *  \par Function Description
  *  This function saves the data in a libgeda format to a file
+ *
+ *  \bug g_access introduces a race condition in certain cases, but
+ *  solves bug #698565 in the normal use-case
+ *
  *  \param [in] toplevel    The current TOPLEVEL.
  *  \param [in] object_list The head of a GList of OBJECTs to save.
  *  \param [in] filename    The filename to save the data to.
+ *  \param [in,out] err     #GError structure for error reporting.
  *  \return 1 on success, 0 on failure.
  */
 int o_save (TOPLEVEL *toplevel, const GList *object_list,
-            const char *filename)
+            const char *filename, GError **err)
 {
-  FILE *fp;
   char *buffer;
 
-  fp = fopen(filename, "wb");
-
-  if (fp == NULL) {
-    s_log_message(_("o_save: Could not open [%s]\n"), filename);
-    return 0;
+  /* Check to see if real filename is writable; if file doesn't exists
+     we assume all is well */
+  if (g_file_test(filename, G_FILE_TEST_EXISTS) && 
+      g_access(filename, W_OK) != 0) {
+    g_set_error (err, G_FILE_ERROR, G_FILE_ERROR_PERM,
+                 _("File %s is read-only"), filename);
+    return 0;      
   }
 
   buffer = o_save_buffer (toplevel, object_list);
-  if (fwrite (buffer, strlen(buffer), 1, fp) != 1) {
-    /* An error occurred with fwrite */
-#warning FIXME: What do we do?
+  if (!g_file_set_contents (filename, buffer, strlen(buffer), err)) {
+    g_free (buffer);
+    return 0;
   }
   g_free (buffer);
-  fclose (fp);
 
   return 1;
 }
 
-/*! \brief Save a file
- *  \par Function Description
- *  This function saves the current page in a libgeda format to a file
- *  \param [in] toplevel    The current TOPLEVEL.
- *  \param [in] filename    The filename to save the data to.
- *  \return 1 on success, 0 on failure.
- */
-int o_save_curr_page (TOPLEVEL *toplevel, const char *filename)
-{
-  return o_save (toplevel, s_page_objects (toplevel->page_current),
-                 filename);
-}
-
 /*! \brief Read a memory buffer
  *  \par Function Description
  *  This function reads data in libgeda format from a memory buffer.
@@ -275,35 +268,30 @@ int o_save_curr_page (TOPLEVEL *toplevel, const char *filename)
  */
 GList *o_read_buffer (TOPLEVEL *toplevel, GList *object_list,
                       char *buffer, const int size,
-                      const char *name)
+                      const char *name, GError **err)
 {
-  char *line = NULL;
+  const char *line = NULL;
   TextBuffer *tb = NULL;
 
   char objtype;
   GList *object_list_save=NULL;
   OBJECT *new_obj=NULL;
-  GList *new_obj_list;
+  GList *new_attrs_list;
+  GList *new_object_list = NULL;
   GList *iter;
-  unsigned int release_ver;
-  unsigned int fileformat_ver;
-  unsigned int current_fileformat_ver;
+  unsigned int release_ver = 0;
+  unsigned int fileformat_ver = 0;
+  unsigned int current_fileformat_ver = FILEFORMAT_VERSION;
   int found_pin = 0;
   OBJECT* last_complex = NULL;
   int itemsread = 0;
 
   int embedded_level = 0;
 
-
-  /* fill version with default file format version (the current one) */
-  current_fileformat_ver = FILEFORMAT_VERSION;
-
   g_return_val_if_fail ((buffer != NULL), NULL);
 
   tb = s_textbuffer_new (buffer, size);
 
-  object_list = g_list_reverse (object_list);
-
   while (1) {
 
     line = s_textbuffer_next_line(tb);
@@ -328,114 +316,131 @@ GList *o_read_buffer (TOPLEVEL *toplevel, GList *object_list,
     switch (objtype) {
 
       case(OBJ_LINE):
-        new_obj = o_line_read (toplevel, line, release_ver, fileformat_ver);
-        object_list = g_list_prepend (object_list, new_obj);
+        if ((new_obj = o_line_read (toplevel, line, release_ver, fileformat_ver, err)) == NULL)
+          goto error;
+        new_object_list = g_list_prepend (new_object_list, new_obj);
         break;
 
 
       case(OBJ_NET):
-        new_obj = o_net_read (toplevel, line, release_ver, fileformat_ver);
-        object_list = g_list_prepend (object_list, new_obj);
+        if ((new_obj = o_net_read (toplevel, line, release_ver, fileformat_ver, err)) == NULL)
+          goto error;
+        new_object_list = g_list_prepend (new_object_list, new_obj);
         break;
 
       case(OBJ_BUS):
-        new_obj = o_bus_read (toplevel, line, release_ver, fileformat_ver);
-        object_list = g_list_prepend (object_list, new_obj);
+        if ((new_obj = o_bus_read (toplevel, line, release_ver, fileformat_ver, err)) == NULL)
+          goto error;
+        new_object_list = g_list_prepend (new_object_list, new_obj);
         break;
 
       case(OBJ_BOX):
-        new_obj = o_box_read (toplevel, line, release_ver, fileformat_ver);
-        object_list = g_list_prepend (object_list, new_obj);
+        if ((new_obj = o_box_read (toplevel, line, release_ver, fileformat_ver, err)) == NULL)
+          goto error;
+        new_object_list = g_list_prepend (new_object_list, new_obj);
         break;
 
       case(OBJ_PICTURE):
-        line = g_strdup (line);
-        new_obj = o_picture_read (toplevel, line, tb, release_ver, fileformat_ver);
-        g_free (line);
-        object_list = g_list_prepend (object_list, new_obj);
+        new_obj = o_picture_read (toplevel, line, tb, release_ver, fileformat_ver, err);
+        if (new_obj == NULL)
+          goto error;
+        new_object_list = g_list_prepend (new_object_list, new_obj);
         break;
 
       case(OBJ_CIRCLE):
-        new_obj = o_circle_read (toplevel, line, release_ver, fileformat_ver);
-        object_list = g_list_prepend (object_list, new_obj);
+        if ((new_obj = o_circle_read (toplevel, line, release_ver, fileformat_ver, err)) == NULL)
+	  goto error;
+        new_object_list = g_list_prepend (new_object_list, new_obj);
         break;
 
       case(OBJ_COMPLEX):
       case(OBJ_PLACEHOLDER):
-        new_obj = o_complex_read (toplevel, line, release_ver, fileformat_ver);
-        object_list = g_list_prepend (object_list, new_obj);
+        if ((new_obj = o_complex_read (toplevel, line, release_ver, fileformat_ver, err)) == NULL)
+          goto error;
+        new_object_list = g_list_prepend (new_object_list, new_obj);
 
         /* last_complex is used for verifying symversion attribute */
         last_complex = new_obj;
         break;
 
       case(OBJ_TEXT):
-        line = g_strdup (line);
-        new_obj = o_text_read (toplevel, line, tb, release_ver, fileformat_ver);
-        g_free (line);
-        object_list = g_list_prepend (object_list, new_obj);
+        new_obj = o_text_read (toplevel, line, tb, release_ver, fileformat_ver, err);
+        if (new_obj == NULL)
+          goto error;
+        new_object_list = g_list_prepend (new_object_list, new_obj);
         break;
 
       case(OBJ_PATH):
-        line = g_strdup(line);
-        new_obj = o_path_read (toplevel, line, tb, release_ver, fileformat_ver);
-        g_free (line);
-        object_list = g_list_prepend (object_list, new_obj);
+        new_obj = o_path_read (toplevel, line, tb, release_ver, fileformat_ver, err);
+        if (new_obj == NULL)
+          goto error;
+        new_object_list = g_list_prepend (new_object_list, new_obj);
         break;
 
       case(OBJ_PIN):
-        new_obj = o_pin_read (toplevel, line, release_ver, fileformat_ver);
-        object_list = g_list_prepend (object_list, new_obj);
+        if ((new_obj = o_pin_read (toplevel, line, release_ver, fileformat_ver, err)) == NULL)
+          goto error;
+        new_object_list = g_list_prepend (new_object_list, new_obj);
         found_pin++;
         break;
 
       case(OBJ_ARC):
-        new_obj = o_arc_read (toplevel, line, release_ver, fileformat_ver);
-        object_list = g_list_prepend (object_list, new_obj);
+        if ((new_obj = o_arc_read (toplevel, line, release_ver, fileformat_ver, err)) == NULL)
+          goto error;
+        new_object_list = g_list_prepend (new_object_list, new_obj);
         break;
 
       case(STARTATTACH_ATTR): 
         /* first is the fp */
         /* 2nd is the object to get the attributes */
-        new_obj_list = o_read_attribs (toplevel, NULL, new_obj, tb, release_ver, fileformat_ver);
-        new_obj_list = g_list_reverse (new_obj_list);
-        object_list = g_list_concat (new_obj_list, object_list);
+        if (new_obj != NULL) {
+          o_attrib_freeze_hooks (toplevel, new_obj);
+          new_attrs_list = o_read_attribs (toplevel, new_obj, tb, release_ver, fileformat_ver, err);
+          if (new_attrs_list == NULL)
+            goto error;
+          new_object_list = g_list_concat (new_attrs_list, new_object_list);
+          o_attrib_thaw_hooks (toplevel, new_obj);
 
-        /* by now we have finished reading all the attributes */
-        /* did we just finish attaching to a complex object? */
-        if (last_complex)
-        {
-          /* yes */
-          /* verify symbol version (not file format but rather contents) */
-          o_complex_check_symversion(toplevel, last_complex);
-          last_complex = NULL;
+          /* by now we have finished reading all the attributes */
+          /* did we just finish attaching to a complex object? */
+          if (last_complex)
+          {
+            /* yes */
+            /* verify symbol version (not file format but rather contents) */
+            o_complex_check_symversion(toplevel, last_complex);
+            last_complex = NULL;
+          }
+
+          /* slots only apply to complex objects */
+          if (new_obj != NULL &&
+              (new_obj->type == OBJ_COMPLEX ||
+               new_obj->type == OBJ_PLACEHOLDER)) {
+            s_slot_update_object (toplevel, new_obj);
+          }
+          new_obj = NULL;
         }
-
-        /* slots only apply to complex objects */
-        if (new_obj != NULL &&
-            (new_obj->type == OBJ_COMPLEX ||
-             new_obj->type == OBJ_PLACEHOLDER)) {
-          s_slot_update_object (toplevel, new_obj);
+        else {
+          g_set_error (err, EDA_ERROR, EDA_ERROR_PARSE, _("Read unexpected attach "
+                                                                 "symbol start marker in [%s] :\n>>\n%s<<\n"),
+                       name, line);
+          goto error;
         }
-
-        new_obj = NULL;
         break;
 
       case(START_EMBEDDED):
-        new_obj = object_list->data;
-
         if (new_obj != NULL &&
             (new_obj->type == OBJ_COMPLEX ||
              new_obj->type == OBJ_PLACEHOLDER)) {
 
-          object_list_save = object_list;
-          object_list = new_obj->complex->prim_objs;
+          object_list_save = new_object_list;
+          new_object_list = new_obj->complex->prim_objs;
 
           embedded_level++;
         } else {
-          fprintf(stderr, _("Read unexpected embedded "
-                            "symbol start marker in [%s] :\n>>\n%s<<\n"),
-                          name, line);
+          g_set_error (err, EDA_ERROR, EDA_ERROR_PARSE, _("Read unexpected embedded "
+                                                                 "symbol start marker in [%s] :\n>>\n%s<<\n"),
+                       name, line);
+          goto error;
         }
         break;
 
@@ -443,13 +448,14 @@ GList *o_read_buffer (TOPLEVEL *toplevel, GList *object_list,
         if (embedded_level>0) {
           /* don't do this since objects are already
            * stored/read translated
-           * o_complex_translate_world (toplevel, object_list->x,
-           *                            object_list->y, object_list->complex);
+           * o_complex_translate_world (toplevel, new_object_list->x,
+           *                            new_object_list->y, new_object_list->complex);
            */
+          new_object_list = g_list_reverse (new_object_list);
 
           new_obj = object_list_save->data;
-          new_obj->complex->prim_objs = object_list;
-          object_list = object_list_save;
+          new_obj->complex->prim_objs = new_object_list;
+          new_object_list = object_list_save;
 
           /* set the parent field now */
           for (iter = new_obj->complex->prim_objs;
@@ -462,9 +468,10 @@ GList *o_read_buffer (TOPLEVEL *toplevel, GList *object_list,
 
           embedded_level--;
         } else {
-          fprintf(stderr, _("Read unexpected embedded "
-                            "symbol end marker in [%s] :\n>>\n%s<<\n"),
-                          name, line);
+          g_set_error (err, EDA_ERROR, EDA_ERROR_PARSE, _("Read unexpected embedded "
+                                                                 "symbol end marker in [%s] :\n>>\n%s<<\n"),
+                       name, line);
+          goto error;
         }
         break;
 
@@ -483,6 +490,11 @@ GList *o_read_buffer (TOPLEVEL *toplevel, GList *object_list,
       case(VERSION_CHAR):
         itemsread = sscanf(line, "v %u %u\n", &release_ver, &fileformat_ver);
 
+        if (itemsread == 0) {
+          g_set_error (err, EDA_ERROR, EDA_ERROR_PARSE, "Failed to parse version from buffer.");
+          goto error;
+        }
+
         /* 20030921 was the last version which did not have a fileformat */
         /* version.  The below latter test should not happen, but it is here */
         /* just in in case. */
@@ -497,10 +509,9 @@ GList *o_read_buffer (TOPLEVEL *toplevel, GList *object_list,
         break;
 
       default:
-        fprintf(stderr, _("Read garbage in [%s] :\n>>\n%s<<\n"),
-                name, line);
+        g_set_error (err, EDA_ERROR, EDA_ERROR_PARSE, _("Read garbage in [%s] :\n>>\n%s<<\n"), name, line);
         new_obj = NULL;
-        break;
+        goto error;
     }
 
   }
@@ -516,16 +527,19 @@ GList *o_read_buffer (TOPLEVEL *toplevel, GList *object_list,
 
   if (found_pin) {
     if (release_ver <= VERSION_20020825) {
-      o_pin_update_whichend (toplevel, object_list, found_pin);
+      o_pin_update_whichend (toplevel, new_object_list, found_pin);
     }
   }
 
   tb = s_textbuffer_free(tb);
 
-  object_list = g_list_reverse (object_list);
+  new_object_list = g_list_reverse(new_object_list);
+  object_list = g_list_concat (object_list, new_object_list);
 
   return(object_list);
-
+ error:
+  s_delete_object_glist(toplevel, new_object_list);
+  return NULL;
 }
 
 /*! \brief Read a file
@@ -555,7 +569,7 @@ GList *o_read (TOPLEVEL *toplevel, GList *object_list, char *filename,
   } 
 
   /* Parse file contents */
-  result = o_read_buffer (toplevel, object_list, buffer, size, filename);
+  result = o_read_buffer (toplevel, object_list, buffer, size, filename, err);
   g_free (buffer);
   return result;
 }
