@@ -17,8 +17,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
-#include <config.h>
+/*! \file g_rc.c
+ *  \brief Parse configuration files.
+ *
+ * Contains functions to open, parse and manage gEDA configuration
+ * (RC) files.
+ */
 
+#include <config.h>
+#include <missing.h>
+
+#include <errno.h>
 #include <stdio.h>
 #include <sys/stat.h>
 #include <ctype.h>
@@ -84,7 +93,7 @@ SCM g_rc_mode_general(SCM scmmode,
   SCM_ASSERT (scm_is_string (scmmode), scmmode,
               SCM_ARG1, rc_name);
   
-  mode = SCM_STRING_CHARS (scmmode);
+  mode = scm_to_utf8_string (scmmode);
   
   index = vstbl_lookup_str(table, table_size, mode);
   /* no match? */
@@ -98,242 +107,295 @@ SCM g_rc_mode_general(SCM scmmode,
     *mode_var = vstbl_get_val(table, index);
     ret = SCM_BOOL_T;
   }
-  
+
+  free (mode);
+
   return ret;
 }
 
-
-/*! \brief Reads the gafrc file.
- *  \par Function Description
- *  This is the function which actually reads in the RC file.
- *  First, it looks in a list of previously read RC files.  If the file has
- *  already been read, it just says OK.  After reading the file, it places
- *  the filename in the list of read files.
+/*! \brief Load a system configuration file.
+ * \par Function Description
+ * Attempts to load the system configuration file with basename \a
+ * rcname.  The string "system-" is prefixed to \a rcname.  If \a
+ * rcname is NULL, the default value of "gafrc" is used.
  *
- *  \param [in] toplevel  The TOPLEVEL object.
- *  \param [in] fname      RC file name to read.
- *  \param [in] ok_msg     Message to print if file is read ok.
- *  \param [in] err_msg    Message to print if file read error occurs
- *  \return 1 on success, 0 otherwise.
+ * \param toplevel  The current #TOPLEVEL structure.
+ * \param rcfile    The basename of the configuration file to load, or NULL.
+ * \param err       Return location for errors, or NULL.
+ * \return TRUE on success, FALSE on failure.
  */
-gint g_rc_parse_general(TOPLEVEL *toplevel,
-			const gchar *fname, 
-			const gchar *ok_msg, const gchar *err_msg)
+gboolean
+g_rc_parse_system (TOPLEVEL *toplevel, const gchar *rcname, GError **err)
 {
-  gint found_rc = FALSE;
-  GList *found_rc_filename_element;
+  gchar *sysname = NULL;
+  gboolean status;
 
-  /* First see if fname is in list of previously read RC files. */
-  found_rc_filename_element = g_list_find_custom(toplevel->RC_list,
-                                                 (gconstpointer) fname,
-                                                 (GCompareFunc) strcmp);
-  if (found_rc_filename_element != NULL) {
-    /* We've already read this one in. */
-    s_log_message(_("RC file [%s] already read in.\n"), fname);
-    return 0;
-  }
+  /* Default to gafrc */
+  rcname = (rcname != NULL) ? rcname : "gafrc";
 
-  /* Now try to read in contents of RC file.  */
-  if (access (fname, R_OK) == 0) {
-    g_read_file (fname);
-    found_rc = 1;
-    /* Everything was OK.  Now add this file to list of read RC files. */
-    toplevel->RC_list = g_list_append (toplevel->RC_list,
-                                        g_strdup (fname));
-    s_log_message (ok_msg, fname);
+  sysname = g_strdup_printf ("system-%s", rcname);
+  status = g_rc_parse_local (toplevel, sysname, s_path_sys_config (), err);
+  g_free (sysname);
+  return status;
+}
+
+/*! \brief Load a user configuration file.
+ * \par Function Description
+ * Attempts to load the user configuration file with basename \a
+ * rcname.  If \a rcname is NULL, the default value of "gafrc" is
+ * used.
+ *
+ * \param toplevel  The current #TOPLEVEL structure.
+ * \param rcfile    The basename of the configuration file to load, or NULL.
+ * \param err       Return location for errors, or NULL.
+ * \return TRUE on success, FALSE on failure.
+ */
+gboolean
+g_rc_parse_user (TOPLEVEL *toplevel, const gchar *rcname, GError **err)
+{
+  /* Default to gafrc */
+  rcname = (rcname != NULL) ? rcname : "gafrc";
+
+  return g_rc_parse_local (toplevel, rcname, s_path_user_config (), err);
+}
+
+/*! \brief Load a local configuration file.
+ * \par Function Description
+ * Attempts to load the configuration file with basename \a rcname
+ * corresponding to \a path, reporting errors via \a err.  If \a path
+ * is a directory, looks for a file named \a rcname in that
+ * directory. Otherwise, looks for a file named \a rcname in the same
+ * directory as \a path. If \a path is NULL, looks in the current
+ * directory. If \a rcname is NULL, the default value of "gafrc" is
+ * used.
+ *
+ * \param toplevel  The current #TOPLEVEL structure.
+ * \param rcname    The basename of the configuration file to load, or NULL.
+ * \param path      The path to load a configuration file for, or NULL.
+ * \param err       Return location for errors, or NULL.
+ * \return TRUE on success, FALSE on failure.
+ */
+gboolean
+g_rc_parse_local (TOPLEVEL *toplevel, const gchar *rcname, const gchar *path,
+                  GError **err)
+{
+  gchar *dir = NULL;
+  gchar *rcfile = NULL;
+  gboolean status;
+  g_return_val_if_fail ((toplevel != NULL), FALSE);
+
+  /* Default to gafrc */
+  rcname = (rcname != NULL) ? rcname : "gafrc";
+  /* Default to cwd */
+  path = (path != NULL) ? path : ".";
+
+  /* If path isn't a directory, get the dirname. */
+  if (g_file_test (path, G_FILE_TEST_IS_DIR)) {
+    dir = g_strdup (path);
   } else {
-    found_rc = 0;
-    s_log_message (err_msg, fname);
+    dir = g_path_get_dirname (path);
   }
 
-  return found_rc;
+  rcfile = g_build_filename (dir, rcname, NULL);
+  status = g_rc_parse_file (toplevel, rcfile, err);
+
+  g_free (dir);
+  g_free (rcfile);
+  return status;
 }
 
-
-/*! \brief Parses a system RC file.
- *  \par Function Description
- *  This function wil open and parse a system rc file.
+/*! \brief Mark a configuration file as read.
+ * \par Function Description
+ * If the config file \a filename has not already been loaded, mark it
+ * as loaded and return TRUE, storing \a filename in \a toplevel (\a
+ * filename should not subsequently be freed).  Otherwise, return
+ * FALSE, and set \a err appropriately.
  *
- *  \param [in] toplevel  The TOPLEVEL object.
- *  \param [in] rcname     System RC file name to parse.
- *  \return 1 on success, 0 on failure.
+ * \note Should only be called by g_rc_parse_file().
+ *
+ * \param toplevel  The current #TOPLEVEL structure.
+ * \param filename  The config file name to test.
+ * \param err       Return location for errors, or NULL.
+ * \return TRUE if \a filename not already loaded, FALSE otherwise.
  */
-gint g_rc_parse_system_rc(TOPLEVEL *toplevel, const gchar *rcname)
+static gboolean
+g_rc_try_mark_read (TOPLEVEL *toplevel, gchar *filename, GError **err)
 {
-  gint found_rc;
-  gchar *tmp;
-  char *filename;
-  gchar *ok_msg, *err_msg;
+  GList *found = NULL;
+  g_return_val_if_fail ((toplevel != NULL), FALSE);
+  g_return_val_if_fail ((filename != NULL), FALSE);
 
-  tmp = g_strconcat (s_path_sys_config (),
-                     G_DIR_SEPARATOR_S,
-                     "system-", rcname,
-                     NULL);
-  filename = f_normalize_filename (tmp, NULL);
-  if (filename == NULL) {
-    return 0;
+  /* Test if marked read already */
+  found = g_list_find_custom (toplevel->RC_list, filename,
+                              (GCompareFunc) strcmp);
+  if (found != NULL) {
+    g_set_error (err, EDA_ERROR, EDA_ERROR_RC_TWICE,
+                 _("Config file already loaded"));
+    return FALSE;
   }
 
-  ok_msg  = g_strdup_printf (_("Read system config file [%%s]\n"));
-  err_msg = g_strdup_printf (_("Did not find required system config file [%%s]\n"));
-  found_rc = g_rc_parse_general(toplevel, filename, ok_msg, err_msg);
-
-  g_free(ok_msg);
-  g_free(err_msg);  
-  g_free(tmp);
-  g_free(filename);
-
-  return found_rc;
+  toplevel->RC_list = g_list_append (toplevel->RC_list, filename);
+  /* N.b. don't free name_norm here; it's stored in the TOPLEVEL. */
+  return TRUE;
 }
 
-/*! \brief Parse a RC file in users home directory.
- *  \par Function Description
- *  This function will open and parse a RC file in the users home directory.
+/*! \brief Load a configuration file.
+ * \par Function Description
+ * Load the configuration file \a rcfile, reporting errors via \a err.
  *
- *  \param [in] toplevel  The TOPLEVEL object.
- *  \param [in] rcname     User's RC file name.
- *  \return 1 on success, 0 on failure.
+ * \param toplevel  The current #TOPLEVEL structure.
+ * \param rcfile    The filename of the configuration file to load.
+ * \param err       Return location for errors, or NULL;
+ * \return TRUE on success, FALSE on failure.
  */
-gint g_rc_parse_home_rc(TOPLEVEL *toplevel, const gchar *rcname)
+gboolean
+g_rc_parse_file (TOPLEVEL *toplevel, const gchar *rcfile, GError **err)
 {
-  gint found_rc;
-  gchar *tmp;
-  char *filename;
-  gchar *ok_msg, *err_msg;
+  gchar *name_norm = NULL;
+  GError *tmp_err = NULL;
+  g_return_val_if_fail ((toplevel != NULL), FALSE);
+  g_return_val_if_fail ((rcfile != NULL), FALSE);
 
-  if (s_path_user_config () == NULL) return 0;
+  /* Normalise filename */
+  name_norm = f_normalize_filename (rcfile, &tmp_err);
+  if (name_norm == NULL) goto parse_file_error;
 
-  tmp = g_build_filename (s_path_user_config (), rcname, NULL);
-  filename = f_normalize_filename (tmp, NULL);
-  if (filename == NULL) {
-    return 0;
+  /* Attempt to load the rc file, if it hasn't been loaded already.
+   * If g_rc_try_mark_read() succeeds, it stores name_norm in
+   * toplevel, so we *don't* free it. */
+  if (g_rc_try_mark_read (toplevel, name_norm, &tmp_err)
+      && g_read_file (toplevel, name_norm, &tmp_err)) {
+    s_log_message (_("Parsed config from [%s]\n"), name_norm);
+    return TRUE;
   }
 
-  ok_msg  = g_strdup_printf (_("Read user config file [%%s]\n"));
-  err_msg = g_strdup_printf (_("Did not find optional user config file [%%s]\n"));
-  found_rc = g_rc_parse_general(toplevel, filename, ok_msg, err_msg);
-  
-  g_free(ok_msg);
-  g_free(err_msg);
-  g_free(tmp);
-  g_free(filename);
-
-  return found_rc;
+ parse_file_error:
+  /* Copy tmp_err into err, with a prefixed message. */
+  /*! \todo We should upgrade to GLib >= 2.16 and use
+   * g_propagate_prefixed_error(). */
+  if (err == NULL) {
+    g_error_free (tmp_err);
+  } else {
+    gchar *orig_msg = tmp_err->message;
+    tmp_err->message =
+      g_strdup_printf (_("Unable to parse config from [%s]: %s"),
+                       (name_norm != NULL) ? name_norm : rcfile, orig_msg);
+    g_free (orig_msg);
+    *err = tmp_err;
+  }
+  g_free (name_norm);
+  return FALSE;
 }
 
-/*! \brief Parse rc file in current working directory.
- *  \par Function Description
- *  This function will open and parse a RC file in the current working directory.
- *
- *  \param [in] toplevel  The TOPLEVEL object.
- *  \param [in] rcname     Local directory RC file name.
- *  \return 1 on success, 0 on failure.
- */
-gint g_rc_parse_local_rc(TOPLEVEL *toplevel, const gchar *rcname)
+static void
+g_rc_parse__process_error (GError **err, const gchar *pname)
 {
-  gint found_rc;
-  char *filename;
-  gchar *ok_msg;
-  gchar *err_msg;
+  char *pbase;
 
-  filename = f_normalize_filename (rcname, NULL);
-  if (filename == NULL) {
-    return 0;
+  /* Take no chances; if err was not set for some reason, bail out. */
+  if (*err == NULL) {
+    const gchar *msgl =
+      _("ERROR: An unknown error occurred while parsing configuration files.");
+    s_log_message ("%s\n", msgl);
+    fprintf(stderr, "%s\n", msgl);
+
+  } else {
+    /* Config files are allowed to be missing or skipped; check for
+     * this. */
+    if (g_error_matches (*err, G_FILE_ERROR, G_FILE_ERROR_NOENT) ||
+        g_error_matches (*err, EDA_ERROR, EDA_ERROR_RC_TWICE)) {
+      return;
+    }
+
+    s_log_message (_("ERROR: %s\n"), (*err)->message);
+    fprintf (stderr, _("ERROR: %s\n"), (*err)->message);
   }
 
-  ok_msg  = g_strdup_printf (_("Read local config file [%%s]\n"));
-  err_msg = g_strdup_printf (_("Did not find optional local config file [%%s]\n"));
-  found_rc = g_rc_parse_general(toplevel, filename, ok_msg, err_msg);
-
-  g_free(ok_msg);
-  g_free(err_msg);
-  g_free(filename);
-
-  return found_rc;
-}
-
-/*! \brief Parse a RC file from a specified location.
- *  \par Function Description
- *  This function will open and parse a RC file from a specified location.
- *
- *  \param [in] toplevel  The TOPLEVEL object.
- *  \param [in] rcname     Specified location RC file name.
- *  \return 1 on success, 0 on failure.
- */
-gint g_rc_parse_specified_rc(TOPLEVEL *toplevel, const gchar *rcname)
-{
-  gint found_rc = 0;
-  char *filename;
-  gchar *rcbasename;
-  gchar *ok_msg;
-  gchar *err_msg;
-
-  if (rcname == NULL) {
-    return 0;
-  }
-
-  filename = f_normalize_filename (rcname, NULL);
-  if (filename == NULL) {
-    return 0;
-  }
-
-  rcbasename = g_path_get_basename (rcname);
-
-  ok_msg  = g_strdup_printf (_("Read specified %s file [%%s]\n"),
-                             rcbasename);
-  err_msg = g_strdup_printf (_("Did not find specified %s file [%%s]\n"),
-                             rcbasename);
-  found_rc = g_rc_parse_general(toplevel, filename, ok_msg, err_msg);
-  
-  g_free(ok_msg);
-  g_free(err_msg);
-  g_free(filename);
-  g_free(rcbasename);
-
-  return found_rc;
+  /* g_path_get_basename() allocates memory, but we don't care
+   * because we're about to exit. */
+  pbase = g_path_get_basename (pname);
+  fprintf (stderr, _("ERROR: The %s log may contain more information.\n"),
+           pbase);
+  exit (1);
 }
 
 /*! \brief General RC file parsing function.
- *  \par Function Description
- *  This function will check for System, HOME and Local RC files matching
- *  the rcname input parameter.  If none of those three are found it will
- *  search for the specified_rc_filename.  When none are found it will
- *  call exit(-1) to terminate the program.
+ * \par Function Description
+ * Calls g_rc_parse_handler() with the default error handler. If any
+ * error other than ENOENT occurs while parsing a configuration file,
+ * prints an informative message and calls exit(1).
  *
- *  \param [in] toplevel              The TOPLEVEL object.
- *  \param [in] rcname                 RC file name.
- *  \param [in] specified_rc_filename  Specific location RC file name.
- *  \return calls exit(-1) when no RC file matching either rcname or
- *          specified_rc_filename is found.
+ * \bug libgeda shouldn't call exit() - this function calls
+ *      g_rc_parse__process_error(), which does.
+ *
+ * \warning Since this function may not return, it should only be used
+ * on application startup or when there is no chance of data loss from
+ * an unexpected exit().
+ *
+ * \param [in] toplevel  The current #TOPLEVEL structure.
+ * \param [in] pname     The name of the application (usually argv[0]).
+ * \param [in] rcname    Config file basename, or NULL.
+ * \param [in] rcfile    Specific config file path, or NULL.
  */
-void g_rc_parse(TOPLEVEL *toplevel,
-		const gchar *rcname, const gchar *specified_rc_filename)
+void
+g_rc_parse (TOPLEVEL *toplevel, const gchar *pname,
+            const gchar *rcname, const gchar *rcfile)
 {
-  gint found_rc = 0;
+  g_rc_parse_handler (toplevel, rcname, rcfile,
+                      (ConfigParseErrorFunc) g_rc_parse__process_error,
+                      (void *) pname);
+}
 
-  /* visit rc files in order */
-  /* Changed by SDB 1.2.2005 in response to Peter Kaiser's bug report.
-   * Read gafrc files first */
-  found_rc |= g_rc_parse_system_rc(toplevel, "gafrc");
-  found_rc |= g_rc_parse_home_rc(toplevel, "gafrc");
-  found_rc |= g_rc_parse_local_rc(toplevel, "gafrc");
-  /* continue support for individual rc files for each program.  */
-  found_rc |= g_rc_parse_system_rc(toplevel, rcname);
-  found_rc |= g_rc_parse_home_rc(toplevel, rcname);
-  found_rc |= g_rc_parse_local_rc(toplevel, rcname);
+/*! \brief General RC file parsing function.
+ * \par Function Description
+ * Attempt to load system, user and local (current working directory)
+ * configuration files, first with the default "gafrc" basename and
+ * then with the basename \a rcname, if \a rcname is not NULL.
+ * Additionally, attempt to load configuration from \a rcfile if \a
+ * rcfile is not NULL.
+ *
+ * If an error occurs, calls \a handler with the provided \a user_data
+ * and a GError.
+ *
+ * \see g_rc_parse().
+ *
+ * \param toplevel  The current #TOPLEVEL structure.
+ * \param rcname    Config file basename, or NULL.
+ * \param rcfile    Specific config file path, or NULL.
+ * \param handler   Handler function for config parse errors.
+ * \param user_data Data to be passed to \a handler.
+ */
+void
+g_rc_parse_handler (TOPLEVEL *toplevel,
+                    const gchar *rcname, const gchar *rcfile,
+                    ConfigParseErrorFunc handler, void *user_data)
+{
+  GError *err = NULL;
 
-  /* New fcn introduced by SDB to consolidate this & make it available 
-   * for other programs */
-  found_rc |= g_rc_parse_specified_rc(toplevel, specified_rc_filename);
+#ifdef HANDLER_DISPATCH
+#  error HANDLER_DISPATCH already defined
+#endif
+#define HANDLER_DISPATCH \
+  do { if (err == NULL) break;  handler (&err, user_data);        \
+       g_error_free (err); err = NULL; } while (0)
 
-  /* Oh well, I couldn't find any rcfile, exit! */
-  if (!found_rc) {
-    /*! \todo these two are basically the
-     * same. Inefficient!
-     */
-    s_log_message(_("Could not find any %s file!\n"), rcname);
-    exit(-1);
+  /* Load configuration files in order. */
+  /* First gafrc files. */
+  g_rc_parse_system (toplevel, NULL, &err); HANDLER_DISPATCH;
+  g_rc_parse_user (toplevel, NULL, &err); HANDLER_DISPATCH;
+  g_rc_parse_local (toplevel, NULL, NULL, &err); HANDLER_DISPATCH;
+  /* Next application-specific rcname. */
+  if (rcname != NULL) {
+    g_rc_parse_system (toplevel, rcname, &err); HANDLER_DISPATCH;
+    g_rc_parse_user (toplevel, rcname, &err); HANDLER_DISPATCH;
+    g_rc_parse_local (toplevel, rcname, NULL, &err); HANDLER_DISPATCH;
   }
+  /* Finally, optional additional config file. */
+  if (rcfile != NULL) {
+    g_rc_parse_file (toplevel, rcfile, &err); HANDLER_DISPATCH;
+  }
+
+#undef HANDLER_DISPATCH
 }
 
 /*! \brief
@@ -346,26 +408,32 @@ void g_rc_parse(TOPLEVEL *toplevel,
 SCM g_rc_component_library(SCM path, SCM name)
 {
   gchar *string;
+  char *temp;
   char *namestr = NULL;
 
   SCM_ASSERT (scm_is_string (path), path,
               SCM_ARG1, "component-library");
-  
+
+  scm_dynwind_begin (0);
   if (name != SCM_UNDEFINED) {
     SCM_ASSERT (scm_is_string (name), name,
 		SCM_ARG2, "component-library");
-    namestr = SCM_STRING_CHARS (name);
+    namestr = scm_to_utf8_string (name);
+    scm_dynwind_free(namestr);
   }
-  
+
   /* take care of any shell variables */
-  string = s_expand_env_variables (SCM_STRING_CHARS (path));
+  temp = scm_to_utf8_string (path);
+  string = s_expand_env_variables (temp);
+  scm_dynwind_unwind_handler (g_free, string, SCM_F_WIND_EXPLICITLY);
+  free (temp);
 
   /* invalid path? */
   if (!g_file_test (string, G_FILE_TEST_IS_DIR)) {
     fprintf(stderr,
             "Invalid path [%s] passed to component-library\n",
             string);
-    g_free(string);
+    scm_dynwind_end();
     return SCM_BOOL_F;
   }
 
@@ -380,8 +448,7 @@ SCM g_rc_component_library(SCM path, SCM name)
     g_free(cwd);
   }
 
-  g_free(string);
-
+  scm_dynwind_end();
   return SCM_BOOL_T;
 }
 
@@ -410,25 +477,29 @@ SCM g_rc_component_library_command (SCM listcmd, SCM getcmd,
   SCM_ASSERT (scm_is_string (name), name, SCM_ARG3, 
               "component-library-command");
 
+  scm_dynwind_begin(0);
+
   /* take care of any shell variables */
   /*! \bug this may be a security risk! */
-  tmp_str = scm_to_locale_string (listcmd);
+  tmp_str = scm_to_utf8_string (listcmd);
   lcmdstr = s_expand_env_variables (tmp_str);
+  scm_dynwind_unwind_handler (g_free, lcmdstr, SCM_F_WIND_EXPLICITLY);
   free (tmp_str); /* this should stay as free (allocated from guile) */
 
   /* take care of any shell variables */
   /*! \bug this may be a security risk! */
-  tmp_str = scm_to_locale_string (getcmd);
+  tmp_str = scm_to_utf8_string (getcmd);
   gcmdstr = s_expand_env_variables (tmp_str);
+  scm_dynwind_unwind_handler (g_free, gcmdstr, SCM_F_WIND_EXPLICITLY);
   free (tmp_str); /* this should stay as free (allocated from guile) */
 
-  namestr = scm_to_locale_string (name);
+  namestr = scm_to_utf8_string (name);
 
   src = s_clib_add_command (lcmdstr, gcmdstr, namestr);
 
   free (namestr); /* this should stay as free (allocated from guile) */
-  g_free (lcmdstr);
-  g_free (gcmdstr);
+
+  scm_dynwind_end();
 
   if (src != NULL) return SCM_BOOL_T;
 
@@ -453,84 +524,24 @@ SCM g_rc_component_library_command (SCM listcmd, SCM getcmd,
  */
 SCM g_rc_component_library_funcs (SCM listfunc, SCM getfunc, SCM name)
 {
+  char *namestr;
+  SCM result = SCM_BOOL_F;
+
   SCM_ASSERT (scm_is_true (scm_procedure_p (listfunc)), listfunc, SCM_ARG1,
 	      "component-library-funcs");
   SCM_ASSERT (scm_is_true (scm_procedure_p (getfunc)), getfunc, SCM_ARG2,
 	      "component-library-funcs");
-  SCM_ASSERT (scm_is_string (name), name, SCM_ARG1, 
+  SCM_ASSERT (scm_is_string (name), name, SCM_ARG3, 
 	      "component-library-funcs");
 
-  if (s_clib_add_scm (listfunc, getfunc, SCM_STRING_CHARS (name)) != NULL) {
-    return SCM_BOOL_T;
-  } else {
-    return SCM_BOOL_F;
-  }
-}
+  namestr = scm_to_utf8_string (name);
 
-/*! \todo Finish function description!!!
- *  \brief
- *  \par Function Description
- *
- *  \param [in] path  
- *  \return SCM_BOOL_T on success, SCM_BOOL_F otherwise.
- */
-SCM g_rc_component_library_search(SCM path)
-{
-  gchar *string;
-  GDir *dir;
-  const gchar *entry;
-  
-  SCM_ASSERT (scm_is_string (path), path,
-              SCM_ARG1, "component-library-search");
-
-  /* take care of any shell variables */
-  string = s_expand_env_variables (SCM_STRING_CHARS (path));
-
-  /* invalid path? */
-  if (!g_file_test (string, G_FILE_TEST_IS_DIR)) {
-    fprintf (stderr,
-             "Invalid path [%s] passed to component-library-search\n",
-             string);
-    g_free(string);
-    return SCM_BOOL_F;
+  if (s_clib_add_scm (listfunc, getfunc, namestr) != NULL) {
+    result = SCM_BOOL_T;
   }
 
-  dir = g_dir_open (string, 0, NULL);
-  if (dir == NULL) {
-    fprintf (stderr,
-             "Invalid path [%s] passed to component-library-search\n",
-             string);
-    g_free(string);
-    return SCM_BOOL_F;
-  }
-
-  while ((entry = g_dir_read_name (dir))) {
-    /* don't do . and .. and special case font */
-    if ((g_strcasecmp (entry, ".")    != 0) && 
-        (g_strcasecmp (entry, "..")   != 0) &&
-        (g_strcasecmp (entry, "font") != 0))
-    {
-      gchar *fullpath = g_build_filename (string, entry, NULL);
-
-      if (g_file_test (fullpath, G_FILE_TEST_IS_DIR)) {
-        if (g_path_is_absolute (fullpath)) {
-          s_clib_add_directory (fullpath, NULL);
-        } else {
-          gchar *cwd = g_get_current_dir ();
-          gchar *temp;
-          temp = g_build_filename (cwd, fullpath, NULL);
-          s_clib_add_directory (temp, NULL);
-          g_free(temp);
-          g_free(cwd);
-        }
-      }
-      g_free(fullpath);
-    }
-  }
-
-  g_free(string);
-
-  return SCM_BOOL_T;
+  free (namestr);
+  return result;
 }
 
 /*! \todo Finish function description!!!
@@ -543,12 +554,15 @@ SCM g_rc_component_library_search(SCM path)
 SCM g_rc_source_library(SCM path)
 {
   gchar *string;
+  char *temp;
   
   SCM_ASSERT (scm_is_string (path), path,
               SCM_ARG1, "source-library");
 
   /* take care of any shell variables */
-  string = s_expand_env_variables (SCM_STRING_CHARS (path));
+  temp = scm_to_utf8_string (path);
+  string = s_expand_env_variables (temp);
+  free (temp);
   
   /* invalid path? */
   if (!g_file_test (string, G_FILE_TEST_IS_DIR)) {
@@ -585,6 +599,7 @@ SCM g_rc_source_library(SCM path)
 SCM g_rc_source_library_search(SCM path)
 {
   gchar *string;
+  char *temp;
   GDir *dir;
   const gchar *entry;
   
@@ -592,7 +607,9 @@ SCM g_rc_source_library_search(SCM path)
               SCM_ARG1, "source-library-search");
 
   /* take care of any shell variables */
-  string = s_expand_env_variables (SCM_STRING_CHARS (path));
+  temp = scm_to_utf8_string (path);
+  string = s_expand_env_variables (temp);
+  free (temp);
 
   /* invalid path? */
   if (!g_file_test (string, G_FILE_TEST_IS_DIR)) {
@@ -639,10 +656,44 @@ SCM g_rc_source_library_search(SCM path)
   }
 
   g_free(string);
+  g_dir_close(dir);
 
   return SCM_BOOL_T;
 }
 
+/*!
+ * \brief Get the name of the RC filename being evaluated.
+ * \par Function Description
+ *
+ * Creates a Guile stack object, extracts the topmost frame from that
+ * stack and gets the sourcefile name.
+ *
+ * \returns If the interpreter can resolve the filename, returns a
+ * Scheme object with the full path to the RC file, otherwise #f
+ */
+SCM
+g_rc_rc_filename()
+{
+  SCM stack, frame, source;
+
+  stack = scm_make_stack (SCM_BOOL_T, SCM_EOL);
+  if (scm_is_false (stack)) {
+    return SCM_BOOL_F;
+  }
+
+  frame = scm_stack_ref (stack, scm_from_int(0));
+  if (scm_is_false (frame)) {
+    return SCM_BOOL_F;
+  }
+
+  source = scm_frame_source (frame);
+  if (scm_is_false (source)) {
+    return SCM_BOOL_F;
+  }
+
+  return scm_source_property (source, scm_sym_filename);
+}
+
 /*! \todo Finish function description!!!
  *  \brief
  *  \par Function Description
@@ -666,9 +717,9 @@ SCM g_rc_world_size(SCM width, SCM height, SCM border)
               SCM_ARG3, FUNC_NAME);
   
   /* yes this is legit, we are casing the resulting double to an int */
-  i_width  = (int) (SCM_NUM2DOUBLE (0, width)  * MILS_PER_INCH);
-  i_height = (int) (SCM_NUM2DOUBLE (0, height) * MILS_PER_INCH);
-  i_border = (int) (SCM_NUM2DOUBLE (0, border) * MILS_PER_INCH);
+  i_width  = (int) (scm_to_double (width)  * MILS_PER_INCH);
+  i_height = (int) (scm_to_double (height) * MILS_PER_INCH);
+  i_border = (int) (scm_to_double (border) * MILS_PER_INCH);
 
   PAPERSIZEtoWORLD(i_width, i_height, i_border,
                    &init_right, &init_bottom);
@@ -694,45 +745,51 @@ SCM g_rc_world_size(SCM width, SCM height, SCM border)
  */
 SCM g_rc_untitled_name(SCM name)
 {
+  char *temp;
   SCM_ASSERT (scm_is_string (name), name,
               SCM_ARG1, "untitled-name");
 
   g_free(default_untitled_name);
 
-  default_untitled_name = g_strdup (SCM_STRING_CHARS (name));
+  temp = scm_to_utf8_string (name);
+  default_untitled_name = g_strdup (temp);
+  free (temp);
 
   return SCM_BOOL_T;
 }
 
 
-/*! \todo Finish function description!!!
- *  \brief
- *  \par Function Description
+/*! \brief Add a directory to the Guile load path.
+ * \par Function Description
+ * Prepends \a s_path to the Guile system '%load-path', after
+ * expanding environment variables.
  *
- *  \param [in] path  
- *  \return SCM_BOOL_T on success, SCM_BOOL_F otherwise.
+ *  \param [in] s_path  Path to be added.
+ *  \return SCM_BOOL_T.
  */
-SCM g_rc_scheme_directory(SCM path)
+SCM g_rc_scheme_directory(SCM s_path)
 {
-  gchar *string;
+  char *temp;
+  gchar *expanded;
+  SCM s_load_path_var;
+  SCM s_load_path;
 
-  SCM_ASSERT (scm_is_string (path), path,
+  SCM_ASSERT (scm_is_string (s_path), s_path,
               SCM_ARG1, "scheme-directory");
 
   /* take care of any shell variables */
-  string = s_expand_env_variables (SCM_STRING_CHARS (path));
+  temp = scm_to_utf8_string (s_path);
+  expanded = s_expand_env_variables (temp);
+  s_path = scm_from_utf8_string (expanded);
+  free (temp);
+  g_free (expanded);
 
-  /* invalid path? */
-  if (!g_file_test (string, G_FILE_TEST_IS_DIR)) {
-    fprintf (stderr,
-             "Invalid path [%s] passed to scheme-directory\n",
-             string);
-    g_free(string);
-    return SCM_BOOL_F;
-  }
+  s_load_path_var = scm_c_lookup ("%load-path");
+  s_load_path = scm_variable_ref (s_load_path_var);
+  scm_variable_set_x (s_load_path_var, scm_cons (s_path, s_load_path));
 
-  g_free(default_scheme_directory);
-  default_scheme_directory = string;
+  scm_remember_upto_here_2 (s_load_path_var, s_load_path);
+  scm_remember_upto_here_1 (s_path);
 
   return SCM_BOOL_T;
 }
@@ -747,12 +804,15 @@ SCM g_rc_scheme_directory(SCM path)
 SCM g_rc_bitmap_directory(SCM path)
 {
   gchar *string;
+  char *temp;
 
   SCM_ASSERT (scm_is_string (path), path,
               SCM_ARG1, "bitmap-directory");
   
   /* take care of any shell variables */
-  string = s_expand_env_variables (SCM_STRING_CHARS (path));
+  temp = scm_to_utf8_string (path);
+  string = s_expand_env_variables (temp);
+  free (temp);
 
   /* invalid path? */
   if (!g_file_test (string, G_FILE_TEST_IS_DIR)) {
@@ -778,11 +838,16 @@ SCM g_rc_bitmap_directory(SCM path)
  */
 SCM g_rc_bus_ripper_symname(SCM scmsymname)
 {
+  char *temp;
+
   SCM_ASSERT (scm_is_string (scmsymname), scmsymname,
               SCM_ARG1, "bus-ripper-symname");
 
   g_free(default_bus_ripper_symname);
-  default_bus_ripper_symname = g_strdup (SCM_STRING_CHARS (scmsymname));
+
+  temp = scm_to_utf8_string (scmsymname);
+  default_bus_ripper_symname = g_strdup (temp);
+  free (temp);
 
   return SCM_BOOL_T;
 }
@@ -796,14 +861,18 @@ SCM g_rc_bus_ripper_symname(SCM scmsymname)
  */
 SCM g_rc_postscript_prolog(SCM scmsymname)
 {
+  char *temp;
+
   SCM_ASSERT (scm_is_string (scmsymname), scmsymname,
               SCM_ARG1, "postsript-prolog");
 
   g_free(default_postscript_prolog);
 
   /* take care of any shell variables */
+  temp = scm_to_utf8_string (scmsymname);
   default_postscript_prolog =
-    s_expand_env_variables (SCM_STRING_CHARS (scmsymname));
+    s_expand_env_variables (temp);
+  free (temp);
 
   return SCM_BOOL_T;
 }
@@ -905,11 +974,15 @@ SCM g_rc_always_promote_attributes(SCM attrlist)
   g_list_free(default_always_promote_attributes);
 
   if (scm_is_string (attrlist)) {
+    char *temp;
     s_log_message(_("WARNING: using a string for 'always-promote-attributes'"
 		    " is deprecated. Use a list of strings instead\n"));
 
     /* convert the space separated strings into a GList */
-    attr2 = g_strsplit(SCM_STRING_CHARS (attrlist)," ", 0);
+    temp = scm_to_utf8_string (attrlist);
+    attr2 = g_strsplit(temp," ", 0);
+    free (temp);
+
     for (i=0; attr2[i] != NULL; i++) {
       if (strlen(attr2[i]) > 0) {
 	list = g_list_prepend(list, g_strdup(attr2[i]));
@@ -921,10 +994,13 @@ SCM g_rc_always_promote_attributes(SCM attrlist)
     length = scm_ilength(attrlist);
     /* convert the scm list into a GList */
     for (i=0; i < length; i++) {
+      char *temp;
       SCM_ASSERT(scm_is_string(scm_list_ref(attrlist, scm_from_int(i))), 
 		 scm_list_ref(attrlist, scm_from_int(i)), SCM_ARG1, 
 		 "always-promote-attribute: list element is not a string");
-      attr = g_strdup(SCM_STRING_CHARS(scm_list_ref(attrlist, scm_from_int(i))));
+      temp = scm_to_utf8_string (scm_list_ref (attrlist, scm_from_int (i)));
+      attr = g_strdup(temp);
+      free (temp);
       list = g_list_prepend(list, attr);
     }
   }
@@ -934,6 +1010,27 @@ SCM g_rc_always_promote_attributes(SCM attrlist)
   return SCM_BOOL_T;
 }
 
+/*! \brief Enable the creation of backup files when saving
+ *  \par Function Description
+ *  If enabled then a backup file, of the form 'example.sch~', is created when
+ *  saving a file.
+ *
+ *  \param [in] mode  String. 'enabled' or 'disabled'
+ *  \return           Bool. False if mode is not a valid value; true if it is.
+ *
+ */
+SCM g_rc_make_backup_files(SCM mode)
+{
+  static const vstbl_entry mode_table[] = {
+    {TRUE , "enabled" },
+    {FALSE, "disabled"},
+  };
+
+  RETURN_G_RC_MODE("make-backup-files",
+                  default_make_backup_files,
+                  2);
+}
+
 extern COLOR print_colors[MAX_COLORS];
 
 SCM g_rc_print_color_map (SCM scm_map)
