@@ -31,58 +31,46 @@
 #include <math.h>
 
 #include <gdk-pixbuf/gdk-pixbuf.h>
+#include <gio/gio.h>
 
 #include "libgeda_priv.h"
 
-/*! Default setting for picture draw function. */
-void (*picture_draw_func)() = NULL;
-
 /*! \brief Create picture OBJECT from character string.
  *  \par Function Description
- *  This function will get the description of a picture from the
- *  character string <B>*first_line</B>.
+ *  Parses \a first_line and subsequent lines from \a tb, and returns
+ *  a newly-created picture #OBJECT.
  *
  *  \param [in]  toplevel       The TOPLEVEL object.
  *  \param [in]  first_line      Character string with picture description.
  *  \param [in]  tb              Text buffer to load embedded data from.
  *  \param [in]  release_ver     libgeda release version number.
  *  \param [in]  fileformat_ver  libgeda file format version number.
- *  \return A pointer to the new picture object.
+ *  \return A pointer to the new picture object, or NULL on error.
  */
 OBJECT *o_picture_read (TOPLEVEL *toplevel,
 		       const char *first_line,
 		       TextBuffer *tb,
 		       unsigned int release_ver,
-		       unsigned int fileformat_ver)
+               unsigned int fileformat_ver,
+               GError **err)
 {
   OBJECT *new_obj;
   int x1, y1;
   int width, height, angle;
-  gchar mirrored, embedded;
+  int mirrored, embedded;
   int num_conv;
   gchar type;
-  gchar *line = NULL;
+  const gchar *line = NULL;
   gchar *filename;
-  GdkPixbuf *pixbuf = NULL;
   gchar *file_content = NULL;
   guint file_length = 0;
-  GError *err = NULL;
 
-  num_conv = sscanf(first_line, "%c %d %d %d %d %d %c %c\n",
+  num_conv = sscanf(first_line, "%c %d %d %d %d %d %d %d\n",
 	 &type, &x1, &y1, &width, &height, &angle, &mirrored, &embedded);
   
   if (num_conv != 8) {
-    s_log_message (_("Error reading picture definition line: %s.\n"),
-                   first_line);
-  }
-
-  /* Convert from ascii character to number */
-  if (g_ascii_isdigit(mirrored)) {
-    mirrored -= 0x30;
-  }
-
-  if (g_ascii_isdigit(embedded)) {
-    embedded -= 0x30;
+    g_set_error(err, EDA_ERROR, EDA_ERROR_PARSE, _("Failed to parse picture definition"));
+    return NULL;
   }
 
   if (width == 0 || height == 0) {
@@ -91,18 +79,19 @@ OBJECT *o_picture_read (TOPLEVEL *toplevel,
   }
 
   if ( (mirrored > 1) || (mirrored < 0)) {
-    s_log_message(_("Found a picture with a wrong 'mirrored' parameter: %c.\n"),
+    s_log_message(_("Found a picture with a wrong 'mirrored' parameter: %d.\n"),
 	    mirrored);
     s_log_message(_("Setting mirrored to 0\n"));
     mirrored = 0;
   }
 
   if ( (embedded > 1) || (embedded < 0)) {
-    s_log_message(_("Found a picture with a wrong 'embedded' parameter: %c.\n"),
+    s_log_message(_("Found a picture with a wrong 'embedded' parameter: %d.\n"),
 	    embedded);
     s_log_message(_("Setting embedded to 0\n"));
     embedded = 0;
   }
+
   switch(angle) {
 	
     case(0):
@@ -122,6 +111,13 @@ OBJECT *o_picture_read (TOPLEVEL *toplevel,
   filename = g_strdup(s_textbuffer_next_line(tb));
   filename = remove_last_nl(filename);	
 
+  /* Handle empty filenames */
+  if (strlen (filename) == 0) {
+    s_log_message (_("Found an image with no filename."));
+    g_free (filename);
+    filename = NULL;
+  }
+
   if (embedded == 1) {
     GString *encoded_picture=g_string_new("");
     char finished = 0;
@@ -140,10 +136,10 @@ OBJECT *o_picture_read (TOPLEVEL *toplevel,
     } while (finished == 0);
 
     /* Decode the picture */
-    file_content = s_encoding_base64_decode(encoded_picture->str,
-                                            encoded_picture->len,
-                                            &file_length);
     if (encoded_picture != NULL) {
+      file_content = s_encoding_base64_decode(encoded_picture->str,
+                                              encoded_picture->len,
+                                              &file_length);
       g_string_free (encoded_picture, TRUE);
     }
 
@@ -154,67 +150,26 @@ OBJECT *o_picture_read (TOPLEVEL *toplevel,
       embedded = 0;
     }
   }
-
-  /* If we have embedded data, try loading from the decoded buffer */
-  if (file_content != NULL) {
-    pixbuf = o_picture_pixbuf_from_buffer (file_content, file_length, &err);
-    if (err != NULL) {
-      s_log_message (_("Failed to load image from embedded data [%s]: %s\n"),
-                     filename, err->message);
-      s_log_message (_("Falling back to file loading. Picture unembedded.\n"));
-      g_error_free (err);
-      err = NULL;
-      embedded = 0;
-    }
-  }
-
-  /* If we haven't loaded the pixbuf above, try loading from file */
-  if (pixbuf == NULL) {
-    pixbuf = gdk_pixbuf_new_from_file (filename, &err);
-    if (err != NULL) {
-      s_log_message (_("Failed to load image from file [%s]: %s\n"),
-                     filename, err->message);
-      g_error_free (err);
-      err = NULL;
-    }
-  }
-
-  /* If the pixbuf couldn't be loaded, then try to load a warning picture */
-  if (pixbuf == NULL) {
-    char *temp_filename;
-
-    s_log_message (_("Loading warning picture.\n"));
-    
-    temp_filename = g_build_filename (toplevel->bitmap_directory,
-                                      "gschem-warning.png", NULL);
-    pixbuf = gdk_pixbuf_new_from_file (temp_filename, NULL);
-    if (pixbuf == NULL) {
-      s_log_message( _("Error loading picture from file: %s.\n"),
-                     temp_filename);
-    }      
-    g_free (temp_filename);
-  }
   
   /* create the picture */
   /* The picture is described by its upper left and lower right corner */
-  new_obj = o_picture_new(toplevel, pixbuf,
-                          file_content, file_length, filename,
-                          (double)width / (double)height,
-                          type,
-                          x1, y1+height, x1+width, y1,
-                          angle, mirrored, embedded);
+  new_obj = o_picture_new (toplevel, file_content, file_length, filename,
+                           type,
+                           x1, y1+height, x1+width, y1,
+                           angle, mirrored, embedded);
 
-  /* Don't free file_content, it is now owned by the picture object */
+  g_free (file_content);
+  g_free (filename);
 
   return new_obj;
 }
 
-
 /*! \brief Create a character string representation of a picture OBJECT.
  *  \par Function Description
  *  This function formats a string in the buffer <B>*buff</B> to describe
  *  the picture object <B>*object</B>.
  *
+ *  \param [in] toplevel  a TOPLEVEL structure
  *  \param [in] object  Picture OBJECT to create string from.
  *  \return A pointer to the picture OBJECT character string.
  *
@@ -222,12 +177,13 @@ OBJECT *o_picture_read (TOPLEVEL *toplevel,
  *  Caller must g_free returned character string.
  *
  */
-char *o_picture_save(OBJECT *object)
+char *o_picture_save(TOPLEVEL *toplevel, OBJECT *object)
 {
   int width, height, x1, y1;
   gchar *encoded_picture=NULL;
   gchar *out=NULL;
   guint encoded_picture_length;
+  const gchar *filename = NULL;
 
   /* calculate the width and height of the box */
   width  = abs(object->picture->lower_x - object->picture->upper_x); 
@@ -242,7 +198,7 @@ char *o_picture_save(OBJECT *object)
 #endif
 
   /* Encode the picture if it's embedded */
-  if (object->picture->embedded == 1) {
+  if (o_picture_is_embedded (toplevel, object)) {
     encoded_picture =
       s_encoding_base64_encode( (char *)object->picture->file_content,
                                 object->picture->file_length,
@@ -253,7 +209,11 @@ char *o_picture_save(OBJECT *object)
     }
   }
 
-  if (object->picture->embedded==1 &&
+  /* Cope with null filename */
+  filename = o_picture_get_filename (toplevel, object);
+  if (filename == NULL) filename = "";
+
+  if (o_picture_is_embedded (toplevel, object) &&
       encoded_picture != NULL) {
     out = g_strdup_printf("%c %d %d %d %d %d %c %c\n%s\n%s\n%s", 
 			  object->type,
@@ -261,8 +221,8 @@ char *o_picture_save(OBJECT *object)
 			  object->picture->angle,
 			  /* Convert the (0,1) chars to ASCII */
 			  (object->picture->mirrored)+0x30, 
-			  object->picture->embedded+0x30, 
-			  object->picture->filename,
+			  '1', 
+			  filename,
 			  encoded_picture,
 			  ".");
   }
@@ -273,8 +233,8 @@ char *o_picture_save(OBJECT *object)
 			  object->picture->angle,
 			  /* Convert the (0,1) chars to ASCII */
 			  (object->picture->mirrored)+0x30, 
-			  object->picture->embedded+0x30, 
-			  object->picture->filename);
+			  '0', 
+			  filename);
   }
   g_free(encoded_picture);
 
@@ -282,29 +242,24 @@ char *o_picture_save(OBJECT *object)
 }
 
 
-/*! \brief Create and add picture OBJECT to list.
+/*! \brief Create a picture object.
  *  \par Function Description
  *  This function creates a new object representing a picture.
  *
- *  The picture is described by its upper left corner - <B>x1</B>, <B>y1</B> -
- *  and its lower right corner - <B>x2</B>, <B>y2</B>.
- *  The <B>type</B> parameter must be equal to #OBJ_PICTURE. 
+ *  The picture is described by its upper left corner (\a x1, \a y1)
+ *  and its lower right corner (\a x2, \ay2).  The \a type parameter
+ *  must be equal to #OBJ_PICTURE.
  *
- *  The #OBJECT structure is allocated with the #s_basic_init_object()
- *  function. The structure describing the picture is allocated and
- *  initialized with the parameters given to the function.
+ *  If \a file_content is non-NULL, it must be a pointer to a buffer
+ *  containing raw image data.  If loading data from \a file_content
+ *  is unsuccessful, and \a filename is non-NULL, an image will
+ *  attempt to be loaded from \a filename.  Otherwise, the picture
+ *  object will be initially empty.
  *
  *  \param [in]     toplevel      The TOPLEVEL object.
- *  \param [in]     pixbuf        The GdkPixbuf picture to add.
- *                                A copy of this pixbuf is made.
- *  \param [in]     file_content  Raw data of the image file.
- *                                NULL for non embedded loading. The object
- *                                object takes ownership of this buffer, and it
- *                                should not be free'd by the caller.
+ *  \param [in]     file_content  Raw data of the image file, or NULL.
  *  \param [in]     file_length   Length of raw data buffer
- *  \param [in]     filename      File name backing this picture.
- *                                A copy of this string is made.
- *  \param [in]     ratio         Picture height to width ratio.
+ *  \param [in]     filename      File name backing this picture, or NULL.
  *  \param [in]     type          Must be OBJ_PICTURE.
  *  \param [in]     x1            Upper x coordinate.
  *  \param [in]     y1            Upper y coordinate.
@@ -313,13 +268,13 @@ char *o_picture_save(OBJECT *object)
  *  \param [in]     angle         Picture rotation angle.
  *  \param [in]     mirrored      Whether the image should be mirrored or not.
  *  \param [in]     embedded      Whether the embedded flag should be set or not.
- *  \return A pointer to the new end of the object list.
+ *  \return A pointer to a new picture #OBJECT.
  */
-OBJECT *o_picture_new(TOPLEVEL *toplevel, GdkPixbuf *pixbuf,
-                      gchar *file_content, gsize file_length, char *filename,
-                      double ratio, char type,
-                      int x1, int y1, int x2, int y2, int angle, char mirrored,
-                      char embedded)
+OBJECT *o_picture_new (TOPLEVEL *toplevel,
+                       const gchar *file_content, gsize file_length,
+                       const gchar *filename,
+                       char type, int x1, int y1, int x2, int y2, int angle,
+                       int mirrored, int embedded)
 {
   OBJECT *new_node;
   PICTURE *picture;
@@ -327,26 +282,47 @@ OBJECT *o_picture_new(TOPLEVEL *toplevel, GdkPixbuf *pixbuf,
   /* create the object */
   new_node = s_basic_new_object(type, "picture");
 
-  picture = (PICTURE *) g_malloc(sizeof(PICTURE));
+  picture = (PICTURE *) g_malloc0 (sizeof(PICTURE));
   new_node->picture = picture;
 
   /* describe the picture with its upper left and lower right corner */
-  picture->upper_x = x1;
-  picture->upper_y = y1;
-  picture->lower_x = x2;
-  picture->lower_y = y2;
+  picture->upper_x = (x1 > x2) ? x2 : x1;
+  picture->upper_y = (y1 > y2) ? y1 : y2;
+  picture->lower_x = (x1 > x2) ? x1 : x2;
+  picture->lower_y = (y1 > y2) ? y2 : y1;
 
-  picture->file_content = file_content;
-  picture->file_length  = file_length;
+  picture->pixbuf = NULL;
+  picture->file_content = NULL;
+  picture->file_length = 0;
+
+  picture->ratio = abs ((double) (x1 - x2) / (y1 - y2));
   picture->filename = g_strdup (filename);
-  picture->ratio = ratio;
-  picture->pixbuf = gdk_pixbuf_copy (pixbuf);
   picture->angle = angle;
   picture->mirrored = mirrored;
   picture->embedded = embedded;
 
-  new_node->draw_func = picture_draw_func;
-  new_node->sel_func  = select_func;
+  if (file_content != NULL) {
+    GError *error = NULL;
+    if (!o_picture_set_from_buffer (toplevel, new_node, filename,
+                                    file_content, file_length, &error)) {
+      s_log_message (_("Failed to load buffer image [%s]: %s\n"),
+                     filename, error->message);
+      g_error_free (error);
+
+      /* Force the data into the object anyway, so as to prevent data
+       * loss of embedded images. */
+      picture->file_content = g_memdup (file_content, file_length);
+      picture->file_length = file_length;
+    }
+  }
+  if (picture->pixbuf == NULL && filename != NULL) {
+    GError *error = NULL;
+    if (!o_picture_set_from_file (toplevel, new_node, filename, &error)) {
+      s_log_message (_("Failed to load image from [%s]: %s\n"),
+                     filename, error->message);
+      g_error_free (error);
+    }
+  }
 
   /* compute the bounding picture */
   o_picture_recalc(toplevel, new_node);
@@ -421,6 +397,39 @@ gboolean o_picture_get_position (TOPLEVEL *toplevel, gint *x, gint *y,
   return TRUE;
 }
                  
+
+/*! \brief Get the width/height ratio of an image.
+ * \par Function Description
+
+ * Returns the width/height ratio of picture \a object, taking the
+ * image rotation into account.
+ *
+ * \param toplevel  The current #TOPLEVEL.
+ * \param object    Picture #OBJECT to inspect.
+ * \return width/height ratio for \a object.
+ */
+double
+o_picture_get_ratio (TOPLEVEL *toplevel, OBJECT *object)
+{
+  g_return_val_if_fail (object != NULL, 1);
+  g_return_val_if_fail (object->picture != NULL, 1);
+
+  /* The effective ratio varies depending on the rotation of the
+   * image. */
+  switch (object->picture->angle) {
+  case 0:
+  case 180:
+    return object->picture->ratio;
+  case 90:
+  case 270:
+    return 1.0 / object->picture->ratio;
+  default:
+    g_critical (_("Picture %p has invalid angle %i\n"), object,
+                object->picture->angle);
+  }
+  return 0;
+}
+
 /*! \brief Modify the description of a picture OBJECT.
  *  \par Function Description
  *  This function modifies the coordinates of one of the four corner of
@@ -448,13 +457,15 @@ void o_picture_modify(TOPLEVEL *toplevel, OBJECT *object,
 		      int x, int y, int whichone)
 {
   int tmp;
-  
+  double ratio = o_picture_get_ratio (toplevel, object);
+
+  o_emit_pre_change_notify (toplevel, object);
+
   /* change the position of the selected corner */
   switch(whichone) {
     case PICTURE_UPPER_LEFT:
       object->picture->upper_x = x;
-      tmp = abs(object->picture->upper_x - object->picture->lower_x) / 
-	object->picture->ratio;
+      tmp = abs(object->picture->upper_x - object->picture->lower_x) / ratio;
       if (y < object->picture->lower_y) {
 	tmp = -tmp;
       }
@@ -463,8 +474,7 @@ void o_picture_modify(TOPLEVEL *toplevel, OBJECT *object,
 			
     case PICTURE_LOWER_LEFT:
       object->picture->upper_x = x;
-      tmp = abs(object->picture->upper_x - object->picture->lower_x) / 
-	object->picture->ratio;
+      tmp = abs(object->picture->upper_x - object->picture->lower_x) / ratio;
       if (y > object->picture->upper_y) {
 	tmp = -tmp;
       }
@@ -473,8 +483,7 @@ void o_picture_modify(TOPLEVEL *toplevel, OBJECT *object,
       
     case PICTURE_UPPER_RIGHT:
       object->picture->lower_x = x;
-      tmp = abs(object->picture->upper_x - object->picture->lower_x) / 
-	object->picture->ratio;
+      tmp = abs(object->picture->upper_x - object->picture->lower_x) / ratio;
       if (y < object->picture->lower_y) {
 	tmp = -tmp;
       }
@@ -483,8 +492,7 @@ void o_picture_modify(TOPLEVEL *toplevel, OBJECT *object,
       
     case PICTURE_LOWER_RIGHT:
       object->picture->lower_x = x;
-      tmp = abs(object->picture->upper_x - object->picture->lower_x) / 
-	object->picture->ratio;
+      tmp = abs(object->picture->upper_x - object->picture->lower_x) / ratio;
       if (y > object->picture->upper_y) {
 	tmp = -tmp;
       }
@@ -510,6 +518,38 @@ void o_picture_modify(TOPLEVEL *toplevel, OBJECT *object,
 	
   /* recalculate the screen coords and the boundings */
   o_picture_recalc(toplevel, object);
+  o_emit_change_notify (toplevel, object);
+}
+
+/*! \brief Modify a picture object's coordinates.
+ * \par Function Description
+ * Modifies the coordinates of all four corners of a picture \a
+ * object.  The picture is adjusted to fit the rectangle enclosed by
+ * the points (\a x1, \a y1) and (\a x2, \a y2), and scaled as large
+ * as possible to still fit within that rectangle.
+ *
+ * \param [in]     toplevel current #TOPLEVEL.
+ * \param [in,out] object   picture #OBJECT to be modified.
+ * \param [in]     x1       x coordinate of first corner of box.
+ * \param [in]     y1       y coordinate of first corner of box.
+ * \param [in]     x2       x coordinate of second corner of box.
+ * \param [in]     y2       y coordinate of second corner of box.
+ */
+void
+o_picture_modify_all (TOPLEVEL *toplevel, OBJECT *object,
+                      int x1, int y1, int x2, int y2)
+{
+  o_emit_pre_change_notify (toplevel, object);
+
+  /* Normalise the requested rectangle. */
+  object->picture->lower_x = (x1 > x2) ? x1 : x2;
+  object->picture->lower_y = (y1 > y2) ? y2 : y1;
+  object->picture->upper_x = (x1 > x2) ? x2 : x1;
+  object->picture->upper_y = (y1 > y2) ? y1 : y2;
+
+  /* recalculate the world coords and bounds */
+  o_box_recalc(toplevel, object);
+  o_emit_change_notify (toplevel, object);
 }
 
 /*! \brief Rotate picture OBJECT using WORLD coordinates.
@@ -598,9 +638,17 @@ void o_picture_mirror_world(TOPLEVEL *toplevel,
   int newx1, newy1;
   int newx2, newy2;
 
-  
-  /* Set info in object */
-  object->picture->mirrored = (object->picture->mirrored ^ 1) & 1;
+  /* Set info in object. Sometimes it's necessary to change the
+   * rotation angle as well as the mirror flag. */
+  object->picture->mirrored = !object->picture->mirrored;
+  switch (object->picture->angle) {
+  case 90:
+    object->picture->angle = 270;
+    break;
+  case 270:
+    object->picture->angle = 90;
+    break;
+  }
 
   /* translate object to origin */
   object->picture->upper_x -= world_centerx;
@@ -644,8 +692,6 @@ void o_picture_mirror_world(TOPLEVEL *toplevel,
 void o_picture_translate_world(TOPLEVEL *toplevel,
 			       int dx, int dy, OBJECT *object)
 {
-  if (object == NULL) printf("btw NO!\n");
-
   /* Do world coords */
   object->picture->upper_x = object->picture->upper_x + dx;
   object->picture->upper_y = object->picture->upper_y + dy;
@@ -677,6 +723,7 @@ OBJECT *o_picture_copy(TOPLEVEL *toplevel, OBJECT *object)
   new_node->picture = picture;
 
   new_node->color = object->color;
+  new_node->selectable = object->selectable;
 
   /* describe the picture with its upper left and lower right corner */
   picture->upper_x = object->picture->upper_x;
@@ -685,9 +732,8 @@ OBJECT *o_picture_copy(TOPLEVEL *toplevel, OBJECT *object)
   picture->lower_y = object->picture->lower_y;
 
   if (object->picture->file_content != NULL) {
-    picture->file_content = g_malloc (object->picture->file_length);
-    memcpy (picture->file_content, object->picture->file_content,
-                                   object->picture->file_length);
+    picture->file_content = g_memdup (object->picture->file_content,
+                                      object->picture->file_length);
   } else {
     picture->file_content = NULL;
   }
@@ -699,11 +745,8 @@ OBJECT *o_picture_copy(TOPLEVEL *toplevel, OBJECT *object)
   picture->mirrored    = object->picture->mirrored;
   picture->embedded    = object->picture->embedded;
 
-  /* Copy the picture data */
-  picture->pixbuf = gdk_pixbuf_copy (object->picture->pixbuf);
-
-  new_node->draw_func = object->draw_func;
-  new_node->sel_func  = object->sel_func;
+  /* Get the picture data */
+  picture->pixbuf = o_picture_get_pixbuf (toplevel, object);
 
   /* compute the bounding picture */
   o_picture_recalc(toplevel, new_node);
@@ -724,7 +767,8 @@ OBJECT *o_picture_copy(TOPLEVEL *toplevel, OBJECT *object)
  *  \note
  *  Caller must g_free returned guint8 array.
  */
-guint8 *o_picture_rgb_data(GdkPixbuf *image)
+static guint8 *
+o_picture_rgb_data(GdkPixbuf *image)
 {
   int width = gdk_pixbuf_get_width(image);
   int height = gdk_pixbuf_get_height(image);
@@ -763,7 +807,8 @@ guint8 *o_picture_rgb_data(GdkPixbuf *image)
  *  \note
  *  Caller must g_free returned guint8 array.
  */
-guint8 *o_picture_mask_data(GdkPixbuf *image)
+static guint8 *
+o_picture_mask_data(GdkPixbuf *image)
 {
   guint8 *pixels;
   guint8 *mask;
@@ -792,6 +837,10 @@ guint8 *o_picture_mask_data(GdkPixbuf *image)
  *  This function prints a picture object. The picture is defined by the
  *  coordinates of its upper left corner in (<B>x</B>,<B>y</B>) and its width
  *  and height given by the <B>width</B> and <B>height</B> parameters. 
+ *
+ *  If the picture object was unable to be loaded, prints a crossed
+ *  box of the same dimensions.
+ *
  *  The Postscript document is defined by the file pointer <B>fp</B>.
  *  Function based on the DIA source code (http://www.gnome.org/projects/dia)
  *  and licensed under the GNU GPL version 2.
@@ -809,7 +858,7 @@ void o_picture_print(TOPLEVEL *toplevel, FILE *fp, OBJECT *o_current,
 {
   int x1, y1, x, y;
   int height, width;
-  GdkPixbuf* image = o_current->picture->pixbuf;
+  GdkPixbuf* image = o_picture_get_pixbuf (toplevel, o_current);
   int img_width, img_height, img_rowstride;
   guint8 *rgb_data;
   guint8 *mask_data;
@@ -822,6 +871,22 @@ void o_picture_print(TOPLEVEL *toplevel, FILE *fp, OBJECT *o_current,
   x1 = o_current->picture->upper_x;
   y1 = o_current->picture->upper_y;
 
+  /* If the image failed to load, try to get hold of the fallback
+   * pixbuf. */
+  if (image == NULL) image = o_picture_get_fallback_pixbuf (toplevel);
+  /* If the image failed to load, draw a box in the default color with a
+   * cross in it. */
+  if (image == NULL) {
+    int line_width = (toplevel->line_style == THICK) ? LINE_WIDTH : 2;
+    o_box_print_solid (toplevel, fp, x1, y1, width, height,
+                       DEFAULT_COLOR, line_width, -1, -1, -1, -1);
+    o_line_print_solid (toplevel, fp, x1, y1, x1+width, y1-height,
+                        DEFAULT_COLOR, line_width, -1, -1, -1, -1);
+    o_line_print_solid (toplevel, fp, x1+width, y1, x1, y1-height,
+                        DEFAULT_COLOR, line_width, -1, -1, -1, -1);
+    return;
+  }
+
   img_width = gdk_pixbuf_get_width(image);
   img_rowstride = gdk_pixbuf_get_rowstride(image);
   img_height = gdk_pixbuf_get_height(image);
@@ -867,71 +932,46 @@ void o_picture_print(TOPLEVEL *toplevel, FILE *fp, OBJECT *o_current,
   fprintf(fp, "grestore\n");
   fprintf(fp, "\n");
    
+  g_object_unref (image);
   g_free(rgb_data);
   g_free(mask_data);
 }
 
 
 /*! \brief Embed the image file associated with a picture
- *
- *  \par Function Description
- *  This function reads and embeds image file associated with the picture.
+ * \par Function Description
+ * Verify that a picture has valid data associated with it, and if so,
+ * mark it to be embedded.
  *
  *  \param [in]     toplevel     The TOPLEVEL object.
  *  \param [in]     object       The picture OBJECT to embed
  */
 void o_picture_embed (TOPLEVEL *toplevel, OBJECT *object)
 {
-  GError *err = NULL;
-  GdkPixbuf *pixbuf;
-  gchar *filename;
+  const gchar *filename = o_picture_get_filename (toplevel, object);
+  gchar *basename;
 
-  /* Free any existing embedded data */
-  g_free (object->picture->file_content);
-  object->picture->file_content = NULL;
+  if (o_picture_is_embedded (toplevel, object)) return;
 
-  g_file_get_contents (object->picture->filename,
-                       &object->picture->file_content,
-                       &object->picture->file_length,
-                       &err);
-  if (err != NULL) {
-    s_log_message (_("Failed to load image from file [%s]: %s\n"),
-                   object->picture->filename, err->message);
-    g_error_free (err);
+  if (object->picture->file_content == NULL) {
+    s_log_message (_("Picture [%s] has no image data.\n"), filename);
+    s_log_message (_("Falling back to file loading. Picture is still unembedded.\n"));
+    object->picture->embedded = 0;
     return;
   }
 
   object->picture->embedded = 1;
 
-  pixbuf = o_picture_pixbuf_from_buffer (object->picture->file_content,
-                                         object->picture->file_length,
-                                         &err);
-  if (err != NULL) {
-    s_log_message (_("Failed to load image from embedded data [%s]: %s\n"),
-                   object->picture->filename, err->message);
-    s_log_message (_("Falling back to file loading. Picture unembedded.\n"));
-    g_error_free (err);
-    object->picture->embedded = 0;
-    return;
-  }
-
-  /* Change to the new pixbuf loaded before we embedded. */
-  if (object->picture->pixbuf != NULL)
-    g_object_unref (object->picture->pixbuf);
-
-  object->picture->pixbuf = pixbuf;
-
-  filename = g_path_get_basename(object->picture->filename);
-  s_log_message (_("Picture [%s] has been embedded\n"), filename);
-  g_free(filename);
+  basename = g_path_get_basename (filename);
+  s_log_message (_("Picture [%s] has been embedded\n"), basename);
+  g_free (basename);
 }
 
 
 /*! \brief Unembed a picture, reloading the image from disk
- *
- *  \par Function Description
- *  This function re-reads the image file associated with the picture, and
- *  discards the embeded copy of the file.
+ * \par Function Description
+ * Verify that the file associated with \a object exists on disk and
+ * is usable, and if so, reload the picture and mark it as unembedded.
  *
  *  \param [in]     toplevel     The TOPLEVEL object.
  *  \param [in]     object       The picture OBJECT to unembed
@@ -939,72 +979,26 @@ void o_picture_embed (TOPLEVEL *toplevel, OBJECT *object)
 void o_picture_unembed (TOPLEVEL *toplevel, OBJECT *object)
 {
   GError *err = NULL;
-  GdkPixbuf *pixbuf;
-  gchar *filename;
+  const gchar *filename = o_picture_get_filename (toplevel, object);
+  gchar *basename;
+
+  if (!o_picture_is_embedded (toplevel, object)) return;
+
+  o_picture_set_from_file (toplevel, object, filename, &err);
 
-  pixbuf = gdk_pixbuf_new_from_file (object->picture->filename, &err);
   if (err != NULL) {
     s_log_message (_("Failed to load image from file [%s]: %s\n"),
-                   object->picture->filename, err->message);
+                   filename, err->message);
+    s_log_message (_("Picture is still embedded.\n"));
     g_error_free (err);
     return;
   }
 
-  /* Change to the new pixbuf loaded from the file. */
-  if (object->picture->pixbuf != NULL)
-    g_object_unref(object->picture->pixbuf);
-
-  object->picture->pixbuf = pixbuf;
-
-  g_free (object->picture->file_content);
-  object->picture->file_content = NULL;
-  object->picture->file_length = 0;
   object->picture->embedded = 0;
 
-  filename = g_path_get_basename(object->picture->filename);
-  s_log_message (_("Picture [%s] has been unembedded\n"), filename);
-  g_free(filename);
-}
-
-
-/*! \brief Load a GdkPixbuf from a memory buffer
- *
- *  \par Function Description
- *  This function loads a GdkPixbuf from a memory buffer. The pixbuf
- *  returned already has a reference taken out on the callers behalf.
- *
- *  \param [in]   file_content  The memory buffer containing the image data.
- *  \param [in]   file_length   The size of the image data
- *  \param [out]  err           GError** pointer to return any error messages.
- *  \return  A GdkPixbuf loaded from the image data, or NULL on error.
- */
-
-GdkPixbuf *o_picture_pixbuf_from_buffer (gchar *file_content,
-                                         gsize file_length,
-                                         GError **err)
-{
-  GdkPixbufLoader *loader;
-  GdkPixbuf *pixbuf;
-
-  loader = gdk_pixbuf_loader_new();
-
-  gdk_pixbuf_loader_write (loader, (guchar *)file_content,
-                                             file_length, err);
-  if (err != NULL && *err != NULL)
-    return NULL;
-
-  gdk_pixbuf_loader_close (loader, err);
-  if (err != NULL && *err != NULL)
-    return NULL;
-
-  pixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
-
-  if (pixbuf != NULL)
-    g_object_ref (pixbuf);
-
-  g_object_unref (loader);
-
-  return pixbuf;
+  basename = g_path_get_basename(filename);
+  s_log_message (_("Picture [%s] has been unembedded\n"), basename);
+  g_free(basename);
 }
 
 /*! \brief Calculates the distance between the given point and the closest
@@ -1041,3 +1035,214 @@ double o_picture_shortest_distance (OBJECT *object, int x, int y,
   return sqrt ((dx * dx) + (dy * dy));
 }
 
+/*! \brief Test whether a picture object is embedded.
+ * \par Function Description
+ * Returns TRUE if the picture \a object will have its data embedded
+ * in a schematic or symbol file; returns FALSE if its data will be
+ * obtained from a separate file.
+ *
+ * \param toplevel  The current #TOPLEVEL.
+ * \param object    The picture #OBJECT to inspect.
+ * \return TRUE if \a object is embedded.
+ */
+gboolean
+o_picture_is_embedded (TOPLEVEL *toplevel, OBJECT *object)
+{
+  g_return_val_if_fail (object != NULL, FALSE);
+  g_return_val_if_fail (object->picture != NULL, FALSE);
+
+  return object->picture->embedded;
+}
+
+/*! \brief Get a pixel buffer for a picture object.
+ * \par Function Description
+ * Returns a #GdkPixbuf for the picture object \a object, or NULL if
+ * the picture could not be loaded.
+ *
+ * The returned value should have its reference count decremented with
+ * g_object_unref() when no longer needed.
+ *
+ * \param toplevel  The current #TOPLEVEL.
+ * \param object    The picture #OBJECT to inspect.
+ * \return A #GdkPixbuf for the picture.
+ */
+GdkPixbuf *
+o_picture_get_pixbuf (TOPLEVEL *toplevel, OBJECT *object)
+{
+  g_return_val_if_fail (object != NULL, NULL);
+  g_return_val_if_fail (object->picture != NULL, NULL);
+
+  if (object->picture->pixbuf != NULL) {
+    return g_object_ref (object->picture->pixbuf);
+  } else {
+    return NULL;
+  }
+}
+
+/*! \brief Get the raw image data from a picture object.
+ * \par Function Description
+ * Returns the raw image file data underlying the picture \a object,
+ * or NULL if the picture could not be loaded.
+ *
+ * \param toplevel  The current #TOPLEVEL.
+ * \param object    The picture #OBJECT to inspect.
+ * \param len       Location to store buffer length.
+ * \return A read-only buffer of raw image data.
+ */
+const char *
+o_picture_get_data (TOPLEVEL *toplevel, OBJECT *object,
+                    size_t *len)
+{
+  g_return_val_if_fail (object != NULL, NULL);
+  g_return_val_if_fail (object->picture != NULL, NULL);
+
+  *len = object->picture->file_length;
+  return object->picture->file_content;
+}
+
+/*! \brief Set a picture object's contents from a buffer.
+ * \par Function Description
+ * Sets the contents of the picture \a object by reading image data
+ * from a buffer.  The buffer should be in on-disk format.
+ *
+ * \param toplevel The current #TOPLEVEL.
+ * \param object   The picture #OBJECT to modify.
+ * \param filename The new filename for the picture.
+ * \param data     The new image data buffer.
+ * \param len      The size of the data buffer.
+ * \param error    Location to return error information.
+ * \return TRUE on success, FALSE on failure.
+ */
+gboolean
+o_picture_set_from_buffer (TOPLEVEL *toplevel, OBJECT *object,
+                           const gchar *filename,
+                           const gchar *data, size_t len,
+                           GError **error)
+{
+  GdkPixbuf *pixbuf;
+  GInputStream *stream;
+  gchar *tmp;
+
+  g_return_val_if_fail (toplevel != NULL, FALSE);
+  g_return_val_if_fail (object != NULL, FALSE);
+  g_return_val_if_fail (object->picture != NULL, FALSE);
+  g_return_val_if_fail (data != NULL, FALSE);
+
+  /* Check that we can actually load the data before making any
+   * changes to the object. */
+  stream = G_INPUT_STREAM (g_memory_input_stream_new_from_data (data, len, NULL));
+  pixbuf = gdk_pixbuf_new_from_stream (stream, NULL, error);
+  g_object_unref (stream);
+  if (pixbuf == NULL) return FALSE;
+
+  o_emit_pre_change_notify (toplevel, object);
+
+  if (object->picture->pixbuf != NULL) {
+    g_object_unref (object->picture->pixbuf);
+  }
+  object->picture->pixbuf = pixbuf;
+
+  object->picture->ratio = ((double) gdk_pixbuf_get_width(pixbuf) /
+                            gdk_pixbuf_get_height(pixbuf));
+
+  tmp = g_strdup (filename);
+  g_free (object->picture->filename);
+  object->picture->filename = tmp;
+
+  gchar *buf = g_realloc (object->picture->file_content,
+                          len);
+  /* It's possible that these buffers might overlap, because the
+   * library user hates us. */
+  memmove (buf, data, len);
+  object->picture->file_content = buf;
+  object->picture->file_length = len;
+
+  o_emit_change_notify (toplevel, object);
+  return TRUE;
+}
+
+/*! \brief Set a picture object's contents from a file.
+ * \par Function Description
+ * Sets the contents of the picture \a object by reading image data
+ * from a file.
+ *
+ * \param toplevel The current #TOPLEVEL.
+ * \param object   The picture #OBJECT to modify.
+ * \param filename The filename to load image data from.
+ * \param error    Location to return error information.
+ * \return TRUE on success, FALSE on failure.
+ */
+gboolean
+o_picture_set_from_file (TOPLEVEL *toplevel, OBJECT *object,
+                         const gchar *filename,
+                         GError **error)
+{
+  gchar *buf;
+  size_t len;
+  gboolean status;
+
+  g_return_val_if_fail (filename != NULL, FALSE);
+
+  if (!g_file_get_contents (filename, &buf, &len, error)) {
+    return FALSE;
+  }
+
+  status = o_picture_set_from_buffer (toplevel, object, filename,
+                                      buf, len, error);
+  g_free (buf);
+  return status;
+}
+
+/*! \brief Get a picture's corresponding filename.
+ * \par Function Description
+ * Returns the filename associated with the picture \a object.
+ *
+ * \param toplevel The current #TOPLEVEL.
+ * \param object   The picture #OBJECT to inspect.
+ * \return the filename associated with \a object.
+ */
+const gchar *
+o_picture_get_filename (TOPLEVEL *toplevel, OBJECT *object)
+{
+  g_return_val_if_fail (object != NULL, NULL);
+  g_return_val_if_fail (object->picture != NULL, NULL);
+
+  return object->picture->filename;
+}
+
+/*! \brief Get fallback pixbuf for displaying pictures.
+ * \par Function Description
+ * Returns a pixbuf containing the fallback image to be used if a
+ * picture object fails to load.  The returned pixbuf should be freed
+ * with g_object_unref() when no longer needed.
+ *
+ * \return a #GdkPixbuf containing a warning image.
+ */
+GdkPixbuf *
+o_picture_get_fallback_pixbuf (TOPLEVEL *toplevel)
+{
+  static GdkPixbuf *pixbuf = NULL;
+  static gboolean failed = FALSE;
+
+  if (pixbuf == NULL && !failed) {
+    gchar *filename;
+    GError *error = NULL;
+
+    filename = g_build_filename (toplevel->bitmap_directory,
+                                 "gschem-warning.png", NULL);
+    pixbuf = gdk_pixbuf_new_from_file (filename, &error);
+
+    if (pixbuf == NULL) {
+      g_warning ( _("Failed to load fallback image %s: %s.\n"),
+                  filename, error->message);
+      g_error_free (error);
+      failed = TRUE;
+    }
+    g_free (filename);
+  }
+
+  if (failed) return NULL;
+
+  g_assert (GDK_IS_PIXBUF (pixbuf));
+  return g_object_ref (pixbuf);
+}
