@@ -25,6 +25,8 @@
 #include <stdlib.h>
 #endif
 
+#include <math.h>
+
 #include "libgeda_priv.h"
 
 #ifdef HAVE_LIBDMALLOC
@@ -77,6 +79,38 @@ void s_cue_postscript_junction (TOPLEVEL * toplevel, FILE * fp,
   fprintf(fp, "fill\n");
 }
 
+/*! \brief Draw an arrow at the end of a net.
+ */
+void
+s_cue_postscript_arrow (TOPLEVEL *toplevel, FILE *fp,
+                        int x, int y, int dx, int dy)
+{
+  int offset = CUE_BOX_SIZE;
+
+  f_print_set_color (toplevel, fp, JUNCTION_COLOR);
+
+  fprintf (fp, "gsave\n");
+
+  /* We compute a transformation so that the arrowhead is aligned with
+   * the net segment. */
+  double len = hypot (dx, dy);
+  if (len != 0) {
+    double c = dx / len;
+    double s = dy / len;
+    fprintf (fp, "[ %.2f %.2f %.2f %.2f %d %d ] concat\n", c, -s, s, c, x, y);
+  } else {
+    fprintf (fp, "%d %d translate\n", x, y); /* Translate to centre point of the arrow */
+  }
+
+  /* Now draw a simple arrowhead */
+  fprintf (fp, "newpath\n");
+  fprintf (fp, "%d %d moveto\n", -offset, -offset);
+  fprintf (fp, "%d %d lineto\n", -offset, offset);
+  fprintf (fp, "%d %d lineto\n", offset, 0);
+  fprintf (fp, "closepath\n"
+               "fill\n"
+               "grestore\n");
+}
 
 /*! \todo Finish function documentation!!!
  *  \brief
@@ -170,13 +204,21 @@ void s_cue_output_lowlevel(TOPLEVEL * toplevel, OBJECT * object, int whichone,
   switch (type) {
 
     case (CONN_ENDPOINT):
-      if (object->type == OBJ_NET) {	/* only nets have these cues */
+      if (object->type == OBJ_NET || object->type == OBJ_PIN) {
         if (count < 1) {	/* Didn't find anything connected there */
-          if (output_type == POSTSCRIPT) {
-            s_cue_postscript_fillbox(toplevel, fp, x, y);
+          if ((object->type == OBJ_NET)
+              && o_net_is_fully_connected (toplevel, object)) {
+            /* Probably connected, so draw friendly arrowhead. The
+             * additional parameters are needed to allow the arrowhead
+             * to be pointed in the direction of the net. */
+            if (output_type == POSTSCRIPT) {
+              s_cue_postscript_arrow (toplevel, fp, x, y,
+                                      object->line->x[whichone] - object->line->x[!whichone],
+                                      object->line->y[whichone] - object->line->y[!whichone]);
+            }
+          } else {
+            s_cue_postscript_fillbox (toplevel, fp, x, y);
           }
-
-
         } else if (count >= 2) {
           if (output_type == POSTSCRIPT)
             s_cue_postscript_junction (toplevel, fp, x, y, bus_involved);
@@ -232,25 +274,39 @@ void s_cue_output_lowlevel_midpoints(TOPLEVEL * toplevel, OBJECT * object,
   }
 }
 
-/*! \todo Finish function documentation!!!
- *  \brief
- *  \par Function Description
+/*! \brief Output cues for a single object
  *
+ *  \par Function Description
+ *  Cues are drawn on pins, nets and buses.
+ *  Two types of cues are drawn:
+ *   - endpoint cues (identifying unconnected ends of objects)
+ *   - junction cues (identifying net/pin/bus junctions)
+ *
+ *  \param [in] toplevel   The TOPLEVEL object
+ *  \param [in] object     The OBJECT to output cues for
+ *  \param [in] fp         The file handle to output to
+ *  \param [in] type       The type of output being produced
  */
 void s_cue_output_single(TOPLEVEL * toplevel, OBJECT * object, FILE * fp,
 			 int type)
 {
-  if (!object) {
-    return;
+  g_return_if_fail (object != NULL);
+
+  switch (object->type) {
+    case (OBJ_NET):
+      s_cue_output_lowlevel(toplevel, object, 0, fp, type);
+      s_cue_output_lowlevel(toplevel, object, 1, fp, type);
+      break;
+    case (OBJ_BUS):
+      s_cue_output_lowlevel(toplevel, object, 0, fp, type);
+      s_cue_output_lowlevel(toplevel, object, 1, fp, type);
+      break;
+    case (OBJ_PIN):
+      s_cue_output_lowlevel(toplevel, object, object->whichend, fp, type);
+      break;
+    default:
+      return;
   }
-
-  if (object->type != OBJ_NET && object->type != OBJ_PIN &&
-      object->type != OBJ_BUS) {
-	return;
-      }
-
-  s_cue_output_lowlevel(toplevel, object, 0, fp, type);
-  s_cue_output_lowlevel(toplevel, object, 1, fp, type);
   s_cue_output_lowlevel_midpoints(toplevel, object, fp, type);
 }
 
